# Copyright (c) 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
import base64
import os

from oslo_config import cfg
from oslo_log import log as logging
import six

from networking_arista._i18n import _, _LW
from networking_arista.common import exceptions as arista_exc

LOG = logging.getLogger(__name__)


@six.add_metaclass(abc.ABCMeta)
class AristaRPCWrapperBase(object):
    """Wraps Arista JSON RPC.

    All communications between Neutron and EOS are over JSON RPC.
    EOS - operating system used on Arista hardware
    Command API - JSON RPC API provided by Arista EOS
    """
    def __init__(self):
        self._validate_config()
        self._server_ip = None
        self.region = cfg.CONF.ml2_arista.region_name
        self.sync_interval = cfg.CONF.ml2_arista.sync_interval
        self.conn_timeout = cfg.CONF.ml2_arista.conn_timeout
        self.eapi_hosts = cfg.CONF.ml2_arista.eapi_host.split(',')

        # Indication of CVX availabililty in the driver.
        self._cvx_available = True

        # Reference to SyncService object which is set in AristaDriver
        self.sync_service = None

    def _validate_config(self):
        if cfg.CONF.ml2_arista.get('eapi_host') == '':
            msg = _('Required option eapi_host is not set')
            LOG.error(msg)
            raise arista_exc.AristaConfigError(msg=msg)
        if cfg.CONF.ml2_arista.get('eapi_username') == '':
            msg = _('Required option eapi_username is not set')
            LOG.error(msg)
            raise arista_exc.AristaConfigError(msg=msg)

    def _api_username(self):
        return cfg.CONF.ml2_arista.eapi_username

    def _api_password(self):
        return cfg.CONF.ml2_arista.eapi_password

    def _get_random_name(self, length=10):
        """Returns a base64 encoded name."""
        result = base64.b64encode(os.urandom(10)).translate(None, b'=+/')

        return result if six.PY2 else result.decode('utf-8')

    def _get_cvx_hosts(self):
        cvx = []
        if self._server_ip:
            # If we know the master's IP, let's start with that
            cvx.append(self._server_ip)

        for h in self.eapi_hosts:
            if h.strip() not in cvx:
                cvx.append(h.strip())

        return cvx

    def set_cvx_unavailable(self):
        self._cvx_available = False
        if self.sync_service:
            self.sync_service.force_sync()

    def set_cvx_available(self):
        self._cvx_available = True

    def cvx_available(self):
        return self._cvx_available

    def check_cvx_availability(self):
        try:
            if self._get_eos_master():
                self.set_cvx_available()
                return True
        except Exception as exc:
            LOG.warning(_LW('%s when getting CVX master'), exc)
            LOG.warning("Failed to initialize connection with CVX. Please "
                        "ensure CVX is reachable and running EOS 4.18.1 "
                        "or greater.")

        self.set_cvx_unavailable()
        return False



# Copyright (c) 2013 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hashlib
import itertools
import mock
from oslo_config import cfg

from neutron.tests import base
from neutron.tests.unit.db import test_db_base_plugin_v2
from neutron_lib import context
from neutron_lib.plugins import constants
from neutron_lib.plugins import directory

from networking_arista.common import exceptions as arista_exc
from networking_arista.l3Plugin import arista_l3_driver as arista
from networking_arista.l3Plugin import l3_arista
from networking_arista.tests.unit import utils


def setup_arista_config(value='', vrf=False, mlag=False, vrf_gateway=False):
    cfg.CONF.set_override('primary_l3_host', value, "l3_arista")
    cfg.CONF.set_override('primary_l3_host_username', value, "l3_arista")
    if vrf:
        cfg.CONF.set_override('use_vrf', vrf, "l3_arista")
        if vrf_gateway:
            cfg.CONF.set_override('vrf_default_route', vrf_gateway,
                                  "l3_arista")
    if mlag:
        cfg.CONF.set_override('secondary_l3_host', value, "l3_arista")
        cfg.CONF.set_override('mlag_config', mlag, "l3_arista")


class AristaL3DriverTestCasesDefaultVrf(base.BaseTestCase):
    """Test cases to test the RPC between Arista Driver and EOS.

    Tests all methods used to send commands between Arista L3 Driver and EOS
    to program routing functions in Default VRF.
    """

    def setUp(self):
        super(AristaL3DriverTestCasesDefaultVrf, self).setUp()
        setup_arista_config('value')
        self.drv = arista.AristaL3Driver()
        self.drv._servers = []
        self.drv._servers.append(mock.MagicMock())

    def test_no_exception_on_correct_configuration(self):
        self.assertIsNotNone(self.drv)

    def test_create_router_on_eos(self):
        router_name = 'test-router-1'
        route_domain = '123:123'

        self.drv.create_router_on_eos(router_name, route_domain,
                                      self.drv._servers[0])
        cmds = ['enable', 'configure', 'exit']

        self.drv._servers[0].execute.assert_called_once_with(cmds,
                                                             keep_alive=True)

    def test_delete_router_from_eos(self):
        router_name = 'test-router-1'

        self.drv.delete_router_from_eos(router_name, self.drv._servers[0])
        cmds = ['enable', 'configure', 'exit']

        self.drv._servers[0].execute.assert_called_once_with(cmds,
                                                             keep_alive=True)

    def test_add_interface_to_router_on_eos(self):
        router_name = 'test-router-1'
        segment_id = '123'
        router_ip = '10.10.10.10'
        mask = '255.255.255.0'
        fixed_ip = '10.10.10.15'

        self.drv.add_interface_to_router(segment_id, router_name, fixed_ip,
                                         router_ip, mask,
                                         self.drv._servers[0])
        cmds = ['enable', 'configure', 'ip routing',
                'vlan %s' % segment_id, 'exit',
                'interface vlan %s' % segment_id,
                'ip address %s/%s' % (fixed_ip, mask), 'exit']

        self.drv._servers[0].execute.assert_called_once_with(cmds,
                                                             keep_alive=True)

    def test_delete_interface_from_router_on_eos(self):
        router_name = 'test-router-1'
        segment_id = '123'

        self.drv.delete_interface_from_router(segment_id, router_name,
                                              self.drv._servers[0])
        cmds = ['enable', 'configure', 'no interface vlan %s' % segment_id,
                'exit']

        self.drv._servers[0].execute.assert_called_once_with(cmds,
                                                             keep_alive=True)


class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
    """Test cases to test the RPC between Arista Driver and EOS.

    Tests all methods used to send commands between Arista L3 Driver and EOS
    to program routing functions using multiple VRFs.
    Note that the configuration commands are different when VRFs are used.
    """

    def setUp(self):
        super(AristaL3DriverTestCasesUsingVRFs, self).setUp()
        setup_arista_config('value', vrf=True)
        self.drv = arista.AristaL3Driver()
        self.drv._servers = []
        self.drv._servers.append(mock.MagicMock())

    def test_no_exception_on_correct_configuration(self):
        self.assertIsNotNone(self.drv)

    def test_create_router_on_eos_v2_syntax(self):
        max_vrfs = 5
        routers = ['testRouterV2-%s' % n for n in range(max_vrfs)]
        domains = ['20%s' % n for n in range(max_vrfs)]

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = True
            self.drv._update_vrf_commands()
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        for (r, d) in zip(routers, domains):
            self.drv.create_router_on_eos(r, d, self.drv._servers[0])

            cmds = ['enable', 'configure',
                    'vrf instance %s' % r,
                    'rd %(rd)s:%(rd)s' % {'rd': d}, 'exit',
                    'ip routing vrf %s' % r, 'exit']

            self.drv._servers[0].execute.assert_called_with(cmds,
                                                            keep_alive=True)

    def test_create_router_on_eos_v1_syntax(self):
        max_vrfs = 5
        routers = ['testRouterV1-%s' % n for n in range(max_vrfs)]
        domains = ['10%s' % n for n in range(max_vrfs)]
        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = False
            self.drv._update_vrf_commands()
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        for (r, d) in zip(routers, domains):
            self.drv.create_router_on_eos(r, d, self.drv._servers[0])

            cmds = ['enable', 'configure',
                    'vrf definition %s' % r,
                    'rd %(rd)s:%(rd)s' % {'rd': d}, 'exit',
                    'ip routing vrf %s' % r, 'exit']

            self.drv._servers[0].execute.assert_called_with(cmds,
                                                            keep_alive=True)

    def test_delete_router_from_eos_v1_syntax(self):
        max_vrfs = 5
        routers = ['testRouter-%s' % n for n in range(max_vrfs)]

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = False
            self.drv._update_vrf_commands()
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        for r in routers:
            self.drv.delete_router_from_eos(r, self.drv._servers[0])
            cmds = ['enable', 'configure', 'no vrf definition %s' % r, 'exit']

            self.drv._servers[0].execute.assert_called_with(cmds,
                                                            keep_alive=True)

    def test_delete_router_from_eos_v2_syntax(self):
        max_vrfs = 5
        routers = ['testRouter-%s' % n for n in range(max_vrfs)]

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = True
            self.drv._update_vrf_commands()
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        for r in routers:
            self.drv.delete_router_from_eos(r, self.drv._servers[0])
            cmds = ['enable', 'configure', 'no vrf instance %s' % r, 'exit']

            self.drv._servers[0].execute.assert_called_with(cmds,
                                                            keep_alive=True)

    def test_add_interface_to_router_on_eos_v1(self):
        router_name = 'test-router-1'
        segment_id = '123'
        router_ip = '10.10.10.10'
        mask = '255.255.255.0'
        fixed_ip = '10.10.10.15'

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = False
            self.drv._update_vrf_commands()
            self.drv._select_dicts(4)
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        self.drv.add_interface_to_router(segment_id, router_name, fixed_ip,
                                         router_ip, mask,
                                         self.drv._servers[0])
        cmds = ['enable', 'configure',
                'vlan %s' % segment_id, 'exit',
                'interface vlan %s' % segment_id,
                'vrf forwarding %s' % router_name,
                'ip address %s/%s' % (fixed_ip, mask), 'exit']

        self.drv._servers[0].execute.assert_called_once_with(cmds,
                                                             keep_alive=True)

    def test_add_interface_to_router_on_eos_v2(self):
        router_name = 'test-router-1'
        segment_id = '123'
        router_ip = '10.10.10.10'
        mask = '255.255.255.0'
        fixed_ip = '10.10.10.15'

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = True
            self.drv._update_vrf_commands()
            self.drv._select_dicts(4)
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        self.drv.add_interface_to_router(segment_id, router_name, fixed_ip,
                                         router_ip, mask,
                                         self.drv._servers[0])
        cmds = ['enable', 'configure',
                'vlan %s' % segment_id, 'exit',
                'interface vlan %s' % segment_id,
                'vrf %s' % router_name,
                'ip address %s/%s' % (fixed_ip, mask), 'exit']

        self.drv._servers[0].execute.assert_called_once_with(cmds,
                                                             keep_alive=True)

    def test_delete_interface_from_router_on_eos(self):
        router_name = 'test-router-1'
        segment_id = '123'

        self.drv.delete_interface_from_router(segment_id, router_name,
                                              self.drv._servers[0])
        cmds = ['enable', 'configure', 'no interface vlan %s' % segment_id,
                'exit']

        self.drv._servers[0].execute.assert_called_once_with(cmds,
                                                             keep_alive=True)


class AristaL3DriverTestCasesVRFDefaultGateway(base.BaseTestCase):
    """Test cases to test the RPC between Arista Driver and EOS.

    Tests all methods used to send commands between Arista L3 Driver and EOS
    to program routing functions using multiple VRFs.
    Note that the configuration commands are different when VRFs are used.
    """

    def setUp(self):
        super(AristaL3DriverTestCasesVRFDefaultGateway, self).setUp()
        setup_arista_config('value', vrf=True, vrf_gateway=True)
        self.drv = arista.AristaL3Driver()
        self.drv._servers = []
        self.drv._servers.append(mock.MagicMock())

    def test_no_exception_on_correct_configuration(self):
        self.assertIsNotNone(self.drv)

    def test_create_router_on_eos_v2_syntax(self):
        max_vrfs = 5
        routers = ['testRouterV2-%s' % n for n in range(max_vrfs)]
        domains = ['20%s' % n for n in range(max_vrfs)]

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = True
            self.drv._update_vrf_commands()
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        for (r, d) in zip(routers, domains):
            self.drv.create_router_on_eos(r, d, self.drv._servers[0])

            cmds = ['enable', 'configure',
                    'vrf instance %s' % r,
                    'rd %(rd)s:%(rd)s' % {'rd': d}, 'exit',
                    'ip routing vrf %s' % r, 'exit']

            self.drv._servers[0].execute.assert_called_with(cmds,
                                                            keep_alive=True)

    def test_create_router_on_eos_v1_syntax(self):
        max_vrfs = 5
        routers = ['testRouterV1-%s' % n for n in range(max_vrfs)]
        domains = ['10%s' % n for n in range(max_vrfs)]
        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = False
            self.drv._update_vrf_commands()
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        for (r, d) in zip(routers, domains):
            self.drv.create_router_on_eos(r, d, self.drv._servers[0])

            cmds = ['enable', 'configure',
                    'vrf definition %s' % r,
                    'rd %(rd)s:%(rd)s' % {'rd': d}, 'exit',
                    'ip routing vrf %s' % r, 'exit']

            self.drv._servers[0].execute.assert_called_with(cmds,
                                                            keep_alive=True)

    def test_delete_router_from_eos_v1_syntax(self):
        max_vrfs = 5
        routers = ['testRouter-%s' % n for n in range(max_vrfs)]

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = False
            self.drv._update_vrf_commands()
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        for r in routers:
            self.drv.delete_router_from_eos(r, self.drv._servers[0])
            cmds = ['enable', 'configure', 'no vrf definition %s' % r, 'exit']

            self.drv._servers[0].execute.assert_called_with(cmds,
                                                            keep_alive=True)

    def test_delete_router_from_eos_v2_syntax(self):
        max_vrfs = 5
        routers = ['testRouter-%s' % n for n in range(max_vrfs)]

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = True
            self.drv._update_vrf_commands()
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        for r in routers:
            self.drv.delete_router_from_eos(r, self.drv._servers[0])
            cmds = ['enable', 'configure', 'no vrf instance %s' % r, 'exit']

            self.drv._servers[0].execute.assert_called_with(cmds,
                                                            keep_alive=True)

    def test_add_interface_to_router_on_eos_v1(self):
        router_name = 'test-router-1'
        segment_id = '123'
        router_ip = '10.10.10.10'
        mask = '255.255.255.0'
        fixed_ip = '10.10.10.15'

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = False
            self.drv._update_vrf_commands()
            self.drv._select_dicts(4)
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        self.drv.add_interface_to_router(segment_id, router_name, fixed_ip,
                                         router_ip, mask,
                                         self.drv._servers[0])
        cmds = ['enable', 'configure',
                'vlan %s' % segment_id, 'exit',
                'interface vlan %s' % segment_id,
                'vrf forwarding %s' % router_name,
                'ip address %s/%s' % (fixed_ip, mask),
                'exit']

        self.drv._servers[0].execute.assert_called_once_with(cmds,
                                                             keep_alive=True)

    def test_add_interface_to_router_on_eos_v2(self):
        router_name = 'test-router-1'
        segment_id = '123'
        router_ip = '10.10.10.10'
        mask = '255.255.255.0'
        fixed_ip = '10.10.10.15'

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = True
            self.drv._update_vrf_commands()
            self.drv._select_dicts(4)
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        self.drv.add_interface_to_router(segment_id, router_name, fixed_ip,
                                         router_ip, mask,
                                         self.drv._servers[0])
        cmds = ['enable', 'configure',
                'vlan %s' % segment_id, 'exit',
                'interface vlan %s' % segment_id,
                'vrf %s' % router_name,
                'ip address %s/%s' % (fixed_ip, mask),
                'exit']

        self.drv._servers[0].execute.assert_called_once_with(cmds,
                                                             keep_alive=True)

    def test_delete_interface_from_router_on_eos(self):
        router_name = 'test-router-1'
        segment_id = '123'

        self.drv.delete_interface_from_router(segment_id, router_name,
                                              self.drv._servers[0])
        cmds = ['enable', 'configure', 'no interface vlan %s' % segment_id,
                'exit']

        self.drv._servers[0].execute.assert_called_once_with(cmds,
                                                             keep_alive=True)


class AristaL3DriverTestCasesMlagConfig(base.BaseTestCase):
    """Test cases to test the RPC between Arista Driver and EOS.

    Tests all methods used to send commands between Arista L3 Driver and EOS
    to program routing functions in Default VRF using MLAG configuration.
    MLAG configuration means that the commands will be sent to both
    primary and secondary Arista Switches.
    """

    def setUp(self):
        super(AristaL3DriverTestCasesMlagConfig, self).setUp()
        setup_arista_config('value', mlag=True)
        self.drv = arista.AristaL3Driver()
        self.drv._servers = []
        self.drv._servers.append(mock.MagicMock())
        self.drv._servers.append(mock.MagicMock())

    def test_no_exception_on_correct_configuration(self):
        self.assertIsNotNone(self.drv)

    def test_create_router_on_eos(self):
        router_name = 'test-router-1'
        route_domain = '123:123'
        router_mac = '00:11:22:33:44:55'

        for s in self.drv._servers:
            self.drv.create_router_on_eos(router_name, route_domain, s)
            cmds = ['enable', 'configure',
                    'ip virtual-router mac-address %s' % router_mac, 'exit']

            s.execute.assert_called_with(cmds, keep_alive=True)

    def test_delete_router_from_eos(self):
        router_name = 'test-router-1'

        for s in self.drv._servers:
            self.drv.delete_router_from_eos(router_name, s)
            cmds = ['enable', 'configure', 'exit']

            s.execute.assert_called_once_with(cmds, keep_alive=True)

    def test_add_interface_to_router_on_eos(self):
        router_name = 'test-router-1'
        segment_id = '123'
        router_ip = '10.10.10.10'
        mask = '255.255.255.0'
        fixed_ip = '10.10.10.15'

        for s in self.drv._servers:
            self.drv.add_interface_to_router(segment_id, router_name, fixed_ip,
                                             router_ip, mask, s)
            cmds = ['enable', 'configure', 'ip routing',
                    'vlan %s' % segment_id, 'exit',
                    'interface vlan %s' % segment_id,
                    'ip address %s' % router_ip,
                    'ip virtual-router address %s' % fixed_ip, 'exit']

            s.execute.assert_called_once_with(cmds, keep_alive=True)

    def test_delete_interface_from_router_on_eos(self):
        router_name = 'test-router-1'
        segment_id = '123'

        for s in self.drv._servers:
            self.drv.delete_interface_from_router(segment_id, router_name, s)

            cmds = ['enable', 'configure', 'no interface vlan %s' % segment_id,
                    'exit']

            s.execute.assert_called_once_with(cmds, keep_alive=True)


class AristaL3DriverTestCasesMlagVRFConfig(base.BaseTestCase):
    """Test cases to test the RPC between Arista Driver and EOS.

    Tests all methods used to send commands between Arista L3 Driver and EOS
    to program routing functions in VRFs using MLAG configuration.
    """

    def setUp(self):
        super(AristaL3DriverTestCasesMlagVRFConfig, self).setUp()
        setup_arista_config('value', mlag=True, vrf=True)
        self.drv = arista.AristaL3Driver()
        self.drv._servers = []
        self.drv._servers.append(mock.MagicMock())
        self.drv._servers.append(mock.MagicMock())

    def test_no_exception_on_correct_configuration(self):
        self.assertIsNotNone(self.drv)

    def test_create_router_on_eos(self):
        max_vrfs = 5
        routers = ['testRouter-%s' % n for n in range(max_vrfs)]
        domains = ['10%s' % n for n in range(max_vrfs)]

        router_mac = '00:11:22:33:44:55'

        for s in self.drv._servers:
            for (r, d) in zip(routers, domains):
                self.drv.create_router_on_eos(r, d, s)

                cmds = ['enable', 'configure',
                        'vrf instance %s' % r,
                        'rd %(rd)s:%(rd)s' % {'rd': d},
                        'exit',
                        'ip routing vrf %s' % r,
                        'ip virtual-router mac-address %s' % router_mac,
                        'exit']
                s.execute.assert_called_with(cmds, keep_alive=True)

    def test_delete_router_from_eos(self):
        max_vrfs = 5
        routers = ['testRouter-%s' % n for n in range(max_vrfs)]

        for s in self.drv._servers:
            for r in routers:
                self.drv.delete_router_from_eos(r, s)
                cmds = ['enable', 'configure', 'no vrf instance %s' % r,
                        'exit']

                s.execute.assert_called_with(cmds, keep_alive=True)

    def test_add_interface_to_router_on_eos_v1(self):
        router_name = 'test-router-1'
        segment_id = '123'
        fixed_ip = '10.10.10.15'
        router_ip = '10.10.10.10'
        mask = '255.255.255.0'

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = False
            self.drv._update_vrf_commands()
            self.drv._select_dicts(4)
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        for s in self.drv._servers:
            self.drv.add_interface_to_router(segment_id, router_name, fixed_ip,
                                             router_ip, mask, s)
            cmds = ['enable', 'configure',
                    'vlan %s' % segment_id, 'exit',
                    'interface vlan %s' % segment_id,
                    'vrf forwarding %s' % router_name,
                    'ip address %s' % router_ip,
                    'ip virtual-router address %s' % fixed_ip,
                    'exit']

            s.execute.assert_called_once_with(cmds, keep_alive=True)

    def test_add_interface_to_router_on_eos_v2(self):
        router_name = 'test-router-1'
        segment_id = '123'
        router_ip = '10.10.10.10'
        mask = '255.255.255.0'
        fixed_ip = '10.10.10.15'

        with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
            c.return_value = True
            self.drv._update_vrf_commands()
            self.drv._select_dicts(4)
            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)

        for s in self.drv._servers:
            self.drv.add_interface_to_router(segment_id, router_name, fixed_ip,
                                             router_ip, mask, s)
            cmds = ['enable', 'configure',
                    'vlan %s' % segment_id, 'exit',
                    'interface vlan %s' % segment_id,
                    'vrf %s' % router_name,
                    'ip address %s' % router_ip,
                    'ip virtual-router address %s' % fixed_ip,
                    'exit']

            s.execute.assert_called_once_with(cmds, keep_alive=True)

    def test_delete_interface_from_router_on_eos(self):
        router_name = 'test-router-1'
        segment_id = '123'

        for s in self.drv._servers:
            self.drv.delete_interface_from_router(segment_id, router_name, s)

            cmds = ['enable', 'configure', 'no interface vlan %s' % segment_id,
                    'exit']

            s.execute.assert_called_once_with(cmds, keep_alive=True)


class AristaL3DriverTestCases_v4(base.BaseTestCase):
    """Test cases to test the RPC between Arista Driver and EOS.

    Tests all methods used to send commands between Arista L3 Driver and EOS
    to program routing functions in Default VRF using IPv4.
    """

    def setUp(self):
        super(AristaL3DriverTestCases_v4, self).setUp()
        setup_arista_config('value')
        self.drv = arista.AristaL3Driver()
        self.drv._servers = []
        self.drv._servers.append(mock.MagicMock())

    def test_no_exception_on_correct_configuration(self):
        self.assertIsNotNone(self.drv)

    def test_add_v4_interface_to_router(self):
        gateway_ip = '10.10.10.1'
        cidrs = ['10.10.10.0/24', '10.11.11.0/24']
        fixed_id = '10.10.10.15'

        # Add couple of IPv4 subnets to router
        for cidr in cidrs:
            router = {'id': 'r1',
                      'name': 'test-router-1',
                      'tenant_id': 'ten-a',
                      'seg_id': '123',
                      'cidr': "%s" % cidr,
                      'gip': "%s" % gateway_ip,
                      'ip_version': 4,
                      'fixed_ip': fixed_id}

            self.assertFalse(self.drv.add_router_interface(None, router))

    def test_delete_v4_interface_from_router(self):
        gateway_ip = '10.10.10.1'
        cidrs = ['10.10.10.0/24', '10.11.11.0/24']
        fixed_ip = '10.10.10.15'

        # remove couple of IPv4 subnets from router
        for cidr in cidrs:
            router = {'id': 'r1',
                      'name': 'test-router-1',
                      'tenant_id': 'ten-a',
                      'seg_id': '123',
                      'cidr': "%s" % cidr,
                      'gip': "%s" % gateway_ip,
                      'ip_version': 4,
                      'fixed_id': fixed_ip}

            self.assertFalse(self.drv.remove_router_interface(None, router))


class AristaL3DriverTestCases_v6(base.BaseTestCase):
    """Test cases to test the RPC between Arista Driver and EOS.

    Tests all methods used to send commands between Arista L3 Driver and EOS
    to program routing functions in Default VRF using IPv6.
    """

    def setUp(self):
        super(AristaL3DriverTestCases_v6, self).setUp()
        setup_arista_config('value')
        self.drv = arista.AristaL3Driver()
        self.drv._servers = []
        self.drv._servers.append(mock.MagicMock())

    def test_no_exception_on_correct_configuration(self):
        self.assertIsNotNone(self.drv)

    def test_add_v6_interface_to_router(self):
        gateway_ip = '3FFE::1'
        cidrs = ['3FFE::/16', '2001::/16']
        fixed_ip = '3FFE::5'

        # Add couple of IPv6 subnets to router
        for cidr in cidrs:
            router = {'id': 'r1',
                      'name': 'test-router-1',
                      'tenant_id': 'ten-a',
                      'seg_id': '123',
                      'cidr': "%s" % cidr,
                      'gip': "%s" % gateway_ip,
                      'ip_version': 6,
                      'fixed_ip': fixed_ip}

            self.assertFalse(self.drv.add_router_interface(None, router))

    def test_delete_v6_interface_from_router(self):
        gateway_ip = '3FFE::1'
        cidrs = ['3FFE::/16', '2001::/16']
        fixed_ip = '3FFE::5'

        # remove couple of IPv6 subnets from router
        for cidr in cidrs:
            router = {'id': 'r1',
                      'name': 'test-router-1',
                      'tenant_id': 'ten-a',
                      'seg_id': '123',
                      'cidr': "%s" % cidr,
                      'gip': "%s" % gateway_ip,
                      'ip_version': 6,
                      'fixed_ip': fixed_ip}

            self.assertFalse(self.drv.remove_router_interface(None, router))


class AristaL3DriverTestCases_MLAG_v6(base.BaseTestCase):
    """Test cases to test the RPC between Arista Driver and EOS.

    Tests all methods used to send commands between Arista L3 Driver and EOS
    to program routing functions in Default VRF on MLAG'ed switches using IPv6.
    """

    def setUp(self):
        super(AristaL3DriverTestCases_MLAG_v6, self).setUp()
        setup_arista_config('value', mlag=True)
        self.drv = arista.AristaL3Driver()
        self.drv._servers = []
        self.drv._servers.append(mock.MagicMock())
        self.drv._servers.append(mock.MagicMock())

    def test_no_exception_on_correct_configuration(self):
        self.assertIsNotNone(self.drv)

    def test_add_v6_interface_to_router(self):
        gateway_ip = '3FFE::1'
        cidrs = ['3FFE::/16', '2001::/16']
        fixed_ip = '3FFE::5'

        # Add couple of IPv6 subnets to router
        for cidr in cidrs:
            router = {'id': 'r1',
                      'name': 'test-router-1',
                      'tenant_id': 'ten-a',
                      'seg_id': '123',
                      'cidr': "%s" % cidr,
                      'gip': "%s" % gateway_ip,
                      'ip_version': 6,
                      'fixed_ip': fixed_ip}

            self.assertFalse(self.drv.add_router_interface(None, router))

    def test_delete_v6_interface_from_router(self):
        gateway_ip = '3FFE::1'
        cidrs = ['3FFE::/16', '2001::/16']

        # remove couple of IPv6 subnets from router
        for cidr in cidrs:
            router = {'id': 'r1',
                      'name': 'test-router-1',
                      'tenant_id': 'ten-a',
                      'seg_id': '123',
                      'cidr': "%s" % cidr,
                      'gip': "%s" % gateway_ip,
                      'ip_version': 6}

            self.assertFalse(self.drv.remove_router_interface(None, router))


class AristaL3DriverTestCasesMlag_one_switch_failed(base.BaseTestCase):
    """Test cases to test with non redundant hardare in redundancy mode.

    In the following test cases, the driver is configured in MLAG (redundancy
    mode) but, one of the switches is mocked to throw exceptoin to mimic
    failure of the switch. Ensure that the the operation does not fail when
    one of the switches fails.
    """

    def setUp(self):
        super(AristaL3DriverTestCasesMlag_one_switch_failed, self).setUp()
        setup_arista_config('value', mlag=True)
        self.drv = arista.AristaL3Driver()
        self.drv._servers = []
        self.drv._servers.append(mock.MagicMock())
        self.drv._servers.append(mock.MagicMock())

    def test_create_router_when_one_switch_fails(self):
        router = {}
        router['id'] = 'r1'
        router['name'] = 'test-router-1'

        # Make one of the switches throw an exception - i.e. fail
        self.drv._servers[0].execute = mock.Mock(side_effect=Exception)
        with mock.patch.object(arista.LOG, 'exception') as log_exception:
            self.drv.create_router(None, router)
            log_exception.assert_called_once_with(mock.ANY)

    def test_delete_router_when_one_switch_fails(self):
        router = {}
        router['id'] = 'r1'
        router['name'] = 'test-router-1'
        router_id = '345'

        # Make one of the switches throw an exception - i.e. fail
        self.drv._servers[1].execute = mock.Mock(side_effect=Exception)
        with mock.patch.object(arista.LOG, 'exception') as log_exception:
            self.drv.delete_router(None, router_id, router)
            log_exception.assert_called_once_with(mock.ANY)

    def test_add_router_interface_when_one_switch_fails(self):
        router = {}
        router['id'] = 'r1'
        router['name'] = 'test-router-1'
        router['tenant_id'] = 'ten-1'
        router['seg_id'] = '100'
        router['ip_version'] = 4
        router['cidr'] = '10.10.10.0/24'
        router['gip'] = '10.10.10.1'
        router['fixed_ip'] = '10.10.10.15'

        # Make one of the switches throw an exception - i.e. fail
        self.drv._servers[1].execute = mock.Mock(side_effect=Exception)
        with mock.patch.object(arista.LOG, 'exception') as log_exception:
            self.drv.add_router_interface(None, router)
            log_exception.assert_called_once_with(mock.ANY)

    def test_remove_router_interface_when_one_switch_fails(self):
        router = {}
        router['id'] = 'r1'
        router['name'] = 'test-router-1'
        router['tenant_id'] = 'ten-1'
        router['seg_id'] = '100'
        router['ip_version'] = 4
        router['cidr'] = '10.10.10.0/24'
        router['gip'] = '10.10.10.1'

        # Make one of the switches throw an exception - i.e. fail
        self.drv._servers[0].execute = mock.Mock(side_effect=Exception)
        with mock.patch.object(arista.LOG, 'exception') as log_exception:
            self.drv.remove_router_interface(None, router)
            log_exception.assert_called_once_with(mock.ANY)


class AristaL3ProtectedVlanParserTestCases(base.BaseTestCase):
    """Test cases to test the parsing of protected_vlans config

    1.  Empty string
    2.  Single VLAN
    3.  Single VLAN range
    4.  Multiple VLANs
    5.  Multiple VLAN ranges
    6.  Hybrid VLANs + ranges
    7.  Invalid VLAN
    8.  Range with invalid min
    9.  Range with invalid max
    10. Range with min > max
    11. Non-int VLAN
    12. Non-int min
    13. Non-int max
    """

    def setUp(self):
        super(AristaL3ProtectedVlanParserTestCases, self).setUp()
        setup_arista_config('value')

    def test_empty_string(self):
        cfg.CONF.set_override('protected_vlans', '', 'l3_arista')
        self.drv = arista.AristaL3Driver()
        self.assertEqual(self.drv._protected_vlans, set([1]))

    def test_single_vlan(self):
        cfg.CONF.set_override('protected_vlans', '100', 'l3_arista')
        self.drv = arista.AristaL3Driver()
        self.assertEqual(self.drv._protected_vlans, set([1, 100]))

    def test_single_range(self):
        cfg.CONF.set_override('protected_vlans', '100:105', 'l3_arista')
        self.drv = arista.AristaL3Driver()
        self.assertEqual(self.drv._protected_vlans,
                         set([1] + [i for i in range(100, 106)]))

    def test_multiple_vlans(self):
        cfg.CONF.set_override('protected_vlans', '100,105', 'l3_arista')
        self.drv = arista.AristaL3Driver()
        self.assertEqual(self.drv._protected_vlans, set([1, 100, 105]))

    def test_multiple_ranges(self):
        cfg.CONF.set_override('protected_vlans', '100:105,110:115',
                              'l3_arista')
        self.drv = arista.AristaL3Driver()
        self.assertEqual(self.drv._protected_vlans,
                         set(itertools.chain([1], range(100, 106),
                                             range(110, 116))))

    def test_hybrid_vlan_and_range(self):
        cfg.CONF.set_override('protected_vlans', '100,110:115', 'l3_arista')
        self.drv = arista.AristaL3Driver()
        self.assertEqual(self.drv._protected_vlans,
                         set([1, 100] + list(range(110, 116))))

    def test_vlan_range_inclusive(self):
        cfg.CONF.set_override('protected_vlans', '1,4094', 'l3_arista')
        self.drv = arista.AristaL3Driver()
        self.assertEqual(self.drv._protected_vlans, set([1, 4094]))

    def test_invalid_vlan(self):
        cfg.CONF.set_override('protected_vlans', '5000', 'l3_arista')
        self.assertRaises(arista_exc.AristaServicePluginConfigError,
                          arista.AristaL3Driver)

    def test_invalid_max(self):
        cfg.CONF.set_override('protected_vlans', '100:5000', 'l3_arista')
        self.assertRaises(arista_exc.AristaServicePluginConfigError,
                          arista.AristaL3Driver)

    def test_invalid_min(self):
        cfg.CONF.set_override('protected_vlans', '-100:100', 'l3_arista')
        self.assertRaises(arista_exc.AristaServicePluginConfigError,
                          arista.AristaL3Driver)

    def test_bad_range_bounds(self):
        cfg.CONF.set_override('protected_vlans', '200:100', 'l3_arista')
        self.assertRaises(arista_exc.AristaServicePluginConfigError,
                          arista.AristaL3Driver)

    def test_non_int_vlan(self):
        cfg.CONF.set_override('protected_vlans', 'string', 'l3_arista')
        self.assertRaises(arista_exc.AristaServicePluginConfigError,
                          arista.AristaL3Driver)

    def test_non_int_min(self):
        cfg.CONF.set_override('protected_vlans', 'string:100', 'l3_arista')
        self.assertRaises(arista_exc.AristaServicePluginConfigError,
                          arista.AristaL3Driver)

    def test_non_int_max(self):
        cfg.CONF.set_override('protected_vlans', '100:string', 'l3_arista')
        self.assertRaises(arista_exc.AristaServicePluginConfigError,
                          arista.AristaL3Driver)


class AristaL3SyncWorkerTestBase(
        test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
    """Base test class for L3 Sync Worker test cases"""

    def setUp(self, cleanup=True, vrfDefaultRoute=False):
        cfg.CONF.import_opt('network_vlan_ranges',
                            'neutron.plugins.ml2.drivers.type_vlan',
                            group='ml2_type_vlan')
        cfg.CONF.set_override('network_vlan_ranges', 'default',
                              'ml2_type_vlan')
        cfg.CONF.set_override('enable_cleanup', 'True' if cleanup else 'False',
                              'l3_arista')
        cfg.CONF.set_override('vrf_default_route', str(vrfDefaultRoute),
                              'l3_arista')
        setup_arista_config('value', mlag=True, vrf=True)
        service_plugins = {'arista_l3': 'arista_l3'}
        super(AristaL3SyncWorkerTestBase, self).setUp(
            plugin='ml2',
            service_plugins=service_plugins)
        self.driver = directory.get_plugin(constants.L3)
        self.context = context.get_admin_context()
        self.drv = self.driver.driver
        self.switch1 = utils.MockSwitch()
        self.switch2 = utils.MockSwitch()
        self.switches = [self.switch1, self.switch2]
        self.drv._servers = self.switches
        for worker in self.driver._workers:
            if isinstance(worker, l3_arista.AristaL3SyncWorker):
                self.sync_worker = worker
        self.sync_worker._servers = self.switches

    @staticmethod
    def _get_rd(name):
        hashed = hashlib.sha256(name.encode('utf-8'))
        rdm = str(int(hashed.hexdigest(), 16) % 65536)
        return '%s:%s' % (rdm, rdm)


class AristaL3SyncWorkerCleanupTestCases(AristaL3SyncWorkerTestBase):
    """Test cases to test the L3 Sync Worker with enable_cleanup=True.

    1. Test that VRFs are not cleaned up if router exists
    2. Test that SVIs and VLANs are not cleaned up if router interface exists
    3. Test that stale VRFs are cleaned up if enable_cleanup=True
    4. Test that stale SVIs and VLANs are cleaned up if enable_cleanup=True
    5. Test that stale VRFs are not cleaned up if not name __OpenStack__<...>
    6. Test that stale SVIs and VLANs are not cleaned up if protected
    7. Test that we don't create SVIs or VLANs for router_gateway interfaces
    """

    def setUp(self):
        super(AristaL3SyncWorkerCleanupTestCases, self).setUp()

    def test_router_exists(self):
        router = {'router': {'id': 'r1',
                             'name': 'router1',
                             'tenant_id': 't1',
                             'admin_state_up': True}}
        self.driver.create_router(self.context, router)
        self.sync_worker.synchronize()
        eos_vrf_name = '__OpenStack__r1-router1'
        expected_vrfs = {eos_vrf_name:
                         {'rd': self._get_rd(eos_vrf_name),
                          'routes': {},
                          'svis': []}}
        self.assertEqual(self.switch1._vrfs, expected_vrfs)
        self.assertEqual(self.switch2._vrfs, expected_vrfs)

    def test_router_interface_exists(self):
        router_dict = {'router': {'name': 'router1',
                                  'tenant_id': 't1',
                                  'admin_state_up': True}}
        router = self.driver.create_router(self.context, router_dict)
        net_dict = {'network': {'name': 'n1',
                                'tenant_id': 't1',
                                'admin_state_up': True,
                                'shared': False,
                                'provider:physical_network': 'default',
                                'provider:network_type': 'vlan',
                                'provider:segmentation_id': 100}}
        net = self.plugin.create_network(self.context, net_dict)
        subnet_dict = {'subnet':
                       {'tenant_id': net['tenant_id'],
                        'name': net['name'],
                        'network_id': net['id'],
                        'ip_version': 4,
                        'cidr': '10.0.0.0/24',
                        'gateway_ip': '10.0.0.1',
                        'allocation_pools': None,
                        'enable_dhcp': False,
                        'dns_nameservers': None,
                        'host_routes': None}}
        subnet = self.plugin.create_subnet(self.context, subnet_dict)
        router_interface = {'subnet_id': subnet['id']}
        self.driver.add_router_interface(self.context, router['id'],
                                         router_interface)
        self.sync_worker.synchronize()
        expected_svis_s1 = {'vlan 100': {'ip': '10.0.0.254',
                                         'mask': '24',
                                         'vip': '10.0.0.1'}}
        expected_svis_s2 = {'vlan 100': {'ip': '10.0.0.253',
                                         'mask': '24',
                                         'vip': '10.0.0.1'}}
        expected_vlans = {'100': {'dynamic': False}}
        self.assertEqual(self.switch1._svis, expected_svis_s1)
        self.assertEqual(self.switch2._svis, expected_svis_s2)
        self.assertEqual(self.switch1._vlans, expected_vlans)
        self.assertEqual(self.switch2._vlans, expected_vlans)

    def test_stale_vrf(self):
        eos_vrf_name = '__OpenStack__r1-router1'
        self.switch1._vrfs = {eos_vrf_name:
                              {'rd': self._get_rd(eos_vrf_name),
                               'svis': []}}
        self.switch2._vrfs = {eos_vrf_name:
                              {'rd': self._get_rd(eos_vrf_name),
                               'svis': []}}
        self.sync_worker.synchronize()
        self.assertEqual(self.switch1._vrfs, {})
        self.assertEqual(self.switch2._vrfs, {})

    def test_stale_svi_and_vlan(self):
        self.switch1._svis = {'vlan 100': {'ip': '10.0.0.254',
                                           'mask': '24',
                                           'vip': '10.0.0.1'}}
        self.switch1._vlans = {'100': {'dynamic': False}}
        self.switch2._svis = {'vlan 100': {'ip': '10.0.0.253',
                                           'mask': '24',
                                           'vip': '10.0.0.1'}}
        self.switch2._vlans = {'100': {'dynamic': False}}
        self.sync_worker.synchronize()
        self.assertEqual(self.switch1._svis, {})
        self.assertEqual(self.switch2._svis, {})
        self.assertEqual(self.switch1._vlans, {})
        self.assertEqual(self.switch2._vlans, {})

    def test_non_openstack_vrf(self):
        eos_vrf_name = 'other-vrf'
        expected_vrfs = {eos_vrf_name:
                         {'rd': self._get_rd(eos_vrf_name),
                          'svis': []}}
        self.switch1._vrfs = expected_vrfs
        self.switch2._vrfs = expected_vrfs
        self.sync_worker.synchronize()
        self.assertEqual(self.switch1._vrfs, expected_vrfs)
        self.assertEqual(self.switch2._vrfs, expected_vrfs)

    def test_protected_svi_and_vlan(self):
        self.sync_worker._protected_vlans = set([100])
        protected_svis = {'vlan 100': {'ip': '10.0.0.254',
                                       'mask': '24',
                                       'vip': '10.0.0.1'}}
        protected_vlans = {'100': {'dynamic': False}}
        self.switch1._svis = protected_svis
        self.switch1._vlans = protected_vlans
        self.switch2._svis = protected_svis
        self.switch2._vlans = protected_vlans
        self.sync_worker.synchronize()
        self.assertEqual(self.switch1._svis, protected_svis)
        self.assertEqual(self.switch1._vlans, protected_vlans)
        self.assertEqual(self.switch2._svis, protected_svis)
        self.assertEqual(self.switch2._vlans, protected_vlans)

    def test_router_gw_interface(self):
        router_dict = {'router': {'name': 'router1',
                                  'tenant_id': 't1',
                                  'admin_state_up': True}}
        router = self.driver.create_router(self.context, router_dict)
        router_model = self.driver._get_router(self.context, router['id'])
        net_dict = {'network': {'name': 'n1',
                                'tenant_id': 't1',
                                'admin_state_up': True,
                                'shared': False,
                                'provider:physical_network': 'default',
                                'provider:network_type': 'vlan',
                                'provider:segmentation_id': 100}}
        net = self.plugin.create_network(self.context, net_dict)
        subnet_dict = {'subnet':
                       {'tenant_id': net['tenant_id'],
                        'name': net['name'],
                        'network_id': net['id'],
                        'ip_version': 4,
                        'cidr': '10.0.0.0/24',
                        'gateway_ip': '10.0.0.1',
                        'allocation_pools': None,
                        'enable_dhcp': False,
                        'dns_nameservers': None,
                        'host_routes': None}}
        subnet = self.plugin.create_subnet(self.context, subnet_dict)
        ext_ips = [{'subnet_id': subnet['id'], 'ip_address': '10.0.0.2'}]
        self.driver._create_gw_port(self.context, router['id'], router_model,
                                    net['id'], ext_ips)
        self.sync_worker.synchronize()
        switch1_svi = {'vlan 100':
                       {'ip': '10.0.0.254',
                        'mask': '24',
                        'vip': '10.0.0.2'}
                       }
        switch2_svi = {'vlan 100':
                       {'ip': '10.0.0.253',
                        'mask': '24',
                        'vip': '10.0.0.2'}
                       }
        switch_vlan = {'100': {'dynamic': False}}
        self.assertEqual(self.switch1._svis, switch1_svi)
        self.assertEqual(self.switch2._svis, switch2_svi)
        self.assertEqual(self.switch1._vlans, switch_vlan)
        self.assertEqual(self.switch2._vlans, switch_vlan)


class AristaL3SyncWorkerNoCleanupTestCases(AristaL3SyncWorkerTestBase):
    """Test cases for the L3 Sync Worker with enable_cleanup=False

    1. Test that stale VRFs are not cleaned up if enable_cleanup=False
    2. Test that stale SVIs and VLANs aren't cleaned up if enable_cleanup=False
    """

    def setUp(self):
        super(AristaL3SyncWorkerNoCleanupTestCases, self).setUp(cleanup=False)

    def test_stale_vrf(self):
        eos_vrf_name = '__OpenStack__r1-router1'
        expected_vrfs = {eos_vrf_name:
                         {'rd': self._get_rd(eos_vrf_name),
                          'svis': []}}
        self.switch1._vrfs = expected_vrfs
        self.switch2._vrfs = expected_vrfs
        self.sync_worker.synchronize()
        self.assertEqual(self.switch1._vrfs, expected_vrfs)
        self.assertEqual(self.switch2._vrfs, expected_vrfs)

    def test_stale_svi_and_vlan(self):
        expected_svis = {'vlan 100': {'ip': '10.0.0.254',
                                      'mask': '24',
                                      'vip': '10.0.0.1'}}
        expected_vlans = {'100': {'dynamic': False}}
        self.switch1._svis = expected_svis
        self.switch1._vlans = expected_vlans
        self.switch2._svis = expected_svis
        self.switch2._vlans = expected_vlans
        self.sync_worker.synchronize()
        self.assertEqual(self.switch1._svis, expected_svis)
        self.assertEqual(self.switch1._vlans, expected_vlans)
        self.assertEqual(self.switch2._svis, expected_svis)
        self.assertEqual(self.switch2._vlans, expected_vlans)


class AristaL3SyncWorkerVrfDefaultRouteCleanupTestCases(
        AristaL3SyncWorkerTestBase):
    """Test cases for the L3 Sync Worker vrfDefaultRoute=True

    This test also enabled cleanup=True
    1. Test that default routes are not reset on sync
    """

    def setUp(self):
        super(AristaL3SyncWorkerVrfDefaultRouteCleanupTestCases,
              self).setUp(vrfDefaultRoute=True)

    def test_vrf_default_route_sync(self):
        router_dict = {'router': {'name': 'router1',
                                  'tenant_id': 't1',
                                  'admin_state_up': True}}
        router = self.driver.create_router(self.context, router_dict)
        router_model = self.driver._get_router(self.context, router['id'])
        net_dict = {'network': {'name': 'n1',
                                'tenant_id': 't1',
                                'admin_state_up': True,
                                'shared': False,
                                'provider:physical_network': 'default',
                                'provider:network_type': 'vlan',
                                'provider:segmentation_id': 100}}
        net = self.plugin.create_network(self.context, net_dict)
        subnet_dict = {'subnet':
                       {'tenant_id': net['tenant_id'],
                        'name': net['name'],
                        'network_id': net['id'],
                        'ip_version': 4,
                        'cidr': '10.0.0.0/24',
                        'gateway_ip': '10.0.0.1',
                        'allocation_pools': None,
                        'enable_dhcp': False,
                        'dns_nameservers': None,
                        'host_routes': None}}
        subnet = self.plugin.create_subnet(self.context, subnet_dict)
        ext_ips = [{'subnet_id': subnet['id'], 'ip_address': '10.0.0.2'}]
        self.driver._create_gw_port(self.context, router['id'], router_model,
                                    net['id'], ext_ips)
        self.sync_worker.synchronize()

        switch1_svi = {'vlan 100':
                       {'ip': '10.0.0.254',
                        'mask': '24',
                        'vip': '10.0.0.2'}
                       }
        switch2_svi = {'vlan 100':
                       {'ip': '10.0.0.253',
                        'mask': '24',
                        'vip': '10.0.0.2'}
                       }
        switch_vlan = {'100': {'dynamic': False}}
        self.assertEqual(self.switch1._svis, switch1_svi)
        self.assertEqual(self.switch2._svis, switch2_svi)
        self.assertEqual(self.switch1._vlans, switch_vlan)
        self.assertEqual(self.switch2._vlans, switch_vlan)
        self.sync_worker.synchronize()
        self.switch1.assert_command_not_received('no ip route vrf')
        self.switch2.assert_command_not_received('no ip route vrf')



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import mock

from neutron.db import rbac_db_models as rbac_models
from neutron.tests import base
from neutron.tests.unit import testlib_api
from neutron_lib import constants as n_const
from neutron_lib.plugins import constants as plugin_constants
from neutron_lib.plugins import directory

from oslo_config import cfg
from oslo_utils import importutils

from networking_arista.common import db_lib
import networking_arista.ml2.arista_resources as resources
from networking_arista.tests.unit import utils


class TestAristaResourcesType(resources.AristaResourcesBase):

    f1 = mock.MagicMock()
    f1.return_value = 'data1Prime'

    formatter = [resources.AttributeFormatter('id', 'id'),
                 resources.AttributeFormatter('n_key1', 'c_key1', f1),
                 resources.AttributeFormatter('n_key2', 'c_key2')]


class AristaResourcesClassAttrTest(base.BaseTestCase):
    """Ensure arista resources have class attributes"""

    def test_attributes(self):
        whitelist = ['AristaResourcesBase',
                     'PortResourcesBase',
                     'AttributeFormatter']
        for cls in resources.__dict__.values():
            if (isinstance(cls, type) and
                    cls.__module__ == resources.__name__ and
                    cls.__name__ not in whitelist):
                self.assertIsNotNone(cls.formatter)
                self.assertIsNotNone(cls.id_key)
                self.assertIsNotNone(cls.endpoint)


class AristaResourcesBaseTest(base.BaseTestCase):
    """Test cases for resource manipulation"""

    def setUp(self):
        super(AristaResourcesBaseTest, self).setUp()
        self.rpc = mock.MagicMock()
        self.rpc.region = 'region'

    def test_clear_cvx_data(self):
        # Setup
        ar = resources.AristaResourcesBase(self.rpc)
        ar.cvx_ids = set([i for i in range(10)])
        # Check that clear_cvx_data restores cvx_ids to a NULL set
        ar.clear_cvx_data()
        self.assertEqual(ar.cvx_ids, set())

    def test_clear_neutron_data(self):
        # Setup
        ar = resources.AristaResourcesBase(self.rpc)
        ar.neutron_resources = {i: {'id': i} for i in range(10)}
        # Check that clear_neutron_data restores neutron resources to an
        # empty dict
        ar.clear_neutron_data()
        self.assertEqual(ar.neutron_resources, dict())

    def test_clear_all_data(self):
        # Setup
        ar = resources.AristaResourcesBase(self.rpc)
        ar.cvx_ids = set(i for i in range(10))
        ar.neutron_resources = {i: {'id': i} for i in range(10)}
        # Check that clear_all_data restores neutron resources to an
        # empty dict and cvx_ids to an empty set
        ar.clear_all_data()
        self.assertEqual(ar.neutron_resources, dict())
        self.assertEqual(ar.cvx_ids, set())

    def test_add_neutron_resource(self):
        # Setup
        ar = resources.AristaResourcesBase(self.rpc)
        # Check that the resource is added to neutron_resources
        ar._add_neutron_resource({'id': 1})
        self.assertEqual(ar.neutron_resources, {1: {'id': 1}})

    def test_force_resource_update(self):
        # Setup
        neutron_resources = {i: {'id': i} for i in range(10)}
        ar = resources.AristaResourcesBase(self.rpc)
        ar.cvx_data_stale = False
        ar.neutron_data_stale = False
        ar.cvx_ids = set(range(10))
        ar.neutron_resources = neutron_resources
        resource_to_update = 5
        ar.get_db_resources = mock.MagicMock()
        ar.get_db_resources.return_value = [{'id': resource_to_update}]
        # Ensure that calling force_resource_update would result in that
        # resource being resent to CVX (with any updated data)
        self.assertEqual(ar.resource_ids_to_create(), set())
        ar.force_resource_update(resource_to_update)
        self.assertEqual(ar.resource_ids_to_create(),
                         set([resource_to_update]))

    def test_delete_neutron_resource(self):
        # Setup
        neutron_resources = {i: {'id': i} for i in range(10)}
        ar = resources.AristaResourcesBase(self.rpc)
        ar.get_db_resources = mock.MagicMock()
        ar.get_db_resources.return_value = []
        ar.neutron_resources = copy.copy(neutron_resources)
        id_to_delete = 5
        del neutron_resources[id_to_delete]
        # Delete neutron resource and check that it's no longer in
        # neutron_resources
        ar._delete_neutron_resource(id_to_delete)
        self.assertEqual(ar.neutron_resources, neutron_resources)

    def test_get_endpoint(self):
        # Setup
        regionName = 'region'
        ar = resources.AristaResourcesBase(self.rpc)
        ar.endpoint = '%(region)s'
        self.assertEqual(ar.get_endpoint(), regionName)

    def test_get_resource_ids(self):
        ar = resources.AristaResourcesBase(self.rpc)
        self.assertEqual(ar.get_resource_ids({'id': 1}), set([1]))

    def test_get_cvx_ids(self):
        # Setup
        cvx_ids = range(10)
        ar = resources.AristaResourcesBase(self.rpc)
        ar.endpoint = 'region/%(region)s'
        self.rpc.send_api_request.return_value = [{'id': i} for i in cvx_ids]
        # Check that get_cvx_ids returns the expected value
        self.assertEqual(ar.get_cvx_ids(), set(cvx_ids))
        self.assertEqual(ar.cvx_ids, set(cvx_ids))
        # Check that a second call uses the cached value
        ar.get_cvx_ids()
        self.rpc.send_api_request.assert_called_once()

    def test_get_neutron_ids(self):
        # Setup
        neutron_ids = range(10)
        neutron_resources = [{'id': i} for i in neutron_ids]
        ar = resources.AristaResourcesBase(self.rpc)
        ar.get_db_resources = mock.MagicMock()
        ar.get_db_resources.return_value = neutron_resources
        # Check that get_neutron_resources returns the expected value
        self.assertEqual(ar.get_neutron_ids(), set(neutron_ids))
        # Check that a second call uses the cached value
        ar.get_neutron_ids()
        ar.get_db_resources.assert_called_once()

    def test_get_neutron_resources(self):
        # Setup
        neutron_ids = range(10)
        db_resources = [{'id': i} for i in neutron_ids]
        ar = resources.AristaResourcesBase(self.rpc)
        ar.get_db_resources = mock.MagicMock()
        ar.get_db_resources.return_value = db_resources
        neutron_resources = {i: {'id': i} for i in neutron_ids}
        # Check that get_neutron_resources returns the expected value
        self.assertEqual(ar.get_neutron_resources(), neutron_resources)
        self.assertEqual(ar.neutron_resources, neutron_resources)
        # Check that a second call uses the cached value
        ar.get_neutron_resources()
        ar.get_db_resources.assert_called_once()

    def test_resources_ids_to_delete(self):
        # Setup
        neutron_ids = set(range(7))
        cvx_ids = set(range(3, 10))
        ar = resources.AristaResourcesBase(self.rpc)
        ar.get_cvx_ids = mock.MagicMock()
        ar.get_cvx_ids.return_value = cvx_ids
        ar.get_neutron_ids = mock.MagicMock()
        ar.get_neutron_ids.return_value = neutron_ids
        # Check that the return is the list of ids in cvx but not neutron
        self.assertItemsEqual(ar.resource_ids_to_delete(),
                              cvx_ids - neutron_ids)

    def test_resource_ids_to_create(self):
        # Setup
        cvx_resource_ids = set(range(0, 20, 2))
        neutron_resource_ids = set(range(10))
        ar = resources.AristaResourcesBase(self.rpc)
        ar.cvx_data_stale = False
        ar.neutron_data_stale = False
        ar.cvx_ids = cvx_resource_ids
        ar.neutron_resources = {i: {'id': i} for i in neutron_resource_ids}
        # Ensure that resource ids to create returns the set of ids present in
        # neutron, but not cvx
        self.assertEqual(ar.resource_ids_to_create(), set(range(1, 10, 2)))

    def test_format_for_create(self):
        # Setup
        ar = TestAristaResourcesType(self.rpc)
        neutron_data = [{'id': 'id1',
                         'n_key1': 'data1',
                         'n_key2': 'data2'},
                        {'id': 'id2',
                         'n_key1': 'data1',
                         'n_key2': 'data2',
                         'extra_key': 'data'}]
        expected_cvx_data = [{'id1': {'id': 'id1',
                                      'c_key1': 'data1Prime',
                                      'c_key2': 'data2'}},
                             {'id2': {'id': 'id2',
                                      'c_key1': 'data1Prime',
                                      'c_key2': 'data2'}}]
        test_cases = [(neutron_data[i], expected_cvx_data[i])
                      for i in range(len(neutron_data))]

        # Check that data is correctly formatted for cvx
        for neutron_resource, expected_resource in test_cases:
            formatted_resource = ar.format_for_create(neutron_resource)
            self.assertEqual(formatted_resource, expected_resource)

        # Check that an exception is raised if neutron is missing data that CVX
        # requires
        neutron_data = [{'n_key1': 'data1'},
                        {'n_key1': 'data1',
                         'extra_key': 'data'}]
        for resource in neutron_data:
            self.assertRaises(KeyError, ar.format_for_create, resource)

    def test_format_for_delete(self):
        ar = resources.AristaResourcesBase(self.rpc)
        id_to_format = 1
        self.assertEqual(ar.format_for_delete(id_to_format),
                         {ar.id_key: id_to_format})

    def test_create_cvx_resources(self):
        # Setup
        neutron_ids = set(range(7))
        cvx_ids = set(range(3, 10))
        neutron_resources = [{'id': i} for i in neutron_ids]
        ar = resources.AristaResourcesBase(self.rpc)
        ar.get_db_resources = mock.MagicMock()
        ar.get_db_resources.return_value = neutron_resources
        ar.get_cvx_ids = mock.MagicMock()
        ar.get_cvx_ids.return_value = cvx_ids
        # Check that the return is the resources in neutron but not cvx
        self.assertEqual(ar.create_cvx_resources(),
                         list(neutron_resources[k] for k in
                              neutron_ids - cvx_ids))

    def test_delete_cvx_resources(self):
        # Setup
        neutron_ids = set(range(7))
        cvx_ids = set(range(3, 10))
        neutron_resources = [{'id': i} for i in neutron_ids]
        ar = resources.AristaResourcesBase(self.rpc)
        ar.get_db_resources = mock.MagicMock()
        ar.get_db_resources.return_value = neutron_resources
        ar.cvx_ids = cvx_ids.copy()
        # Check that the return is the list of ids in cvx but not neutron
        self.assertEqual(ar.delete_cvx_resources(),
                         [{'id': i} for i in (cvx_ids - neutron_ids)])


class AristaResourcesTestBase(testlib_api.SqlTestCase):

    def setUp(self):
        super(AristaResourcesTestBase, self).setUp()
        plugin_klass = importutils.import_class(
            "neutron.db.db_base_plugin_v2.NeutronDbPluginV2")
        directory.add_plugin(plugin_constants.CORE, plugin_klass())
        self.rpc = utils.MockCvx('region')

    def run_scenario(self, expect_created):
        utils.setup_scenario()
        # Create resource tests
        resources_created = self.ar.create_cvx_resources()
        self.assertItemsEqual(resources_created,
                              expect_created.values())
        self.ar.clear_cvx_data()

        # Ensure existing resources aren't created
        resources_created = self.ar.create_cvx_resources()
        self.assertEqual(resources_created, [])

        # Delete resource tests
        for resource_id_to_delete in expect_created.keys():
            expect_deleted = [self.ar.format_for_delete(resource_id_to_delete)]
            self.delete_helper(resource_id_to_delete)
            self.ar.clear_all_data()
            resources_deleted = self.ar.delete_cvx_resources()
            self.assertEqual(resources_deleted,
                             expect_deleted)

            # Ensure non-existent resources aren't deleted
            self.ar.clear_all_data()
            resources_deleted = self.ar.delete_cvx_resources()
            self.assertEqual(resources_deleted, [])

    def verify_format_for_create(self, test_cases):
        for neutron_resource, expected_resource in test_cases:
            formatted_resource = self.ar.format_for_create(neutron_resource)
            self.assertEqual(formatted_resource, expected_resource)

    def verify_format_for_delete(self, test_cases):
        for neutron_resource_id, expected_resource in test_cases:
            formatted_resource = self.ar.format_for_delete(neutron_resource_id)
            self.assertEqual(formatted_resource, expected_resource)


class AristaTenantTest(AristaResourcesTestBase):
    """Test cases for creation & deletion of arista tenants"""

    def setUp(self):
        super(AristaTenantTest, self).setUp()
        self.ar = resources.Tenants(self.rpc)

    def delete_helper(self, tenant_id):
        utils.delete_ports_for_tenant(tenant_id)
        utils.delete_segments_for_tenant(tenant_id)
        utils.delete_networks_for_tenant(tenant_id)
        utils.delete_ha_router_for_tenant(tenant_id)

    def test_scenario_tenants(self):
        expect_created = {'t1': {'id': 't1'},
                          't2': {'id': 't2'},
                          'ha-router-project': {'id': 'ha-router-project'}}
        self.run_scenario(expect_created)

    def test_format_tenants(self):
        # format_for_create test setup
        neutron_data = [{'project_id': '1'},
                        {'project_id': '2'}]
        expected_cvx_data = [{'1': {'id': '1'}},
                             {'2': {'id': '2'}}]
        test_cases = [(neutron_data[i], expected_cvx_data[i])
                      for i in range(len(neutron_data))]
        self.verify_format_for_create(test_cases)

    def test_format_tenants_for_delete(self):
        # format_for_delete test setup
        neutron_tenant_id = 't3'
        expected_tenant = {'id': 't3'}
        test_case = [(neutron_tenant_id, expected_tenant)]
        self.verify_format_for_delete(test_case)


class AristaNetworkTest(AristaResourcesTestBase):
    """Test cases for creation & deletion of arista networks"""

    def setUp(self):
        super(AristaNetworkTest, self).setUp()
        self.ar = resources.Networks(self.rpc)

    def delete_helper(self, network_id):
        utils.delete_ports_on_network(network_id)
        utils.delete_segments_for_network(network_id)
        utils.delete_network(network_id)

    def test_networks_scenario(self):
        expect_created = {'n1': {'id': 'n1',
                                 'tenantId': 't1',
                                 'name': 'regular',
                                 'shared': False},
                          'n2': {'id': 'n2',
                                 'tenantId': 't2',
                                 'name': 'hpb',
                                 'shared': False},
                          'HA network': {'id': 'HA network',
                                         'tenantId': 'ha-router-project',
                                         'name': 'l3 ha',
                                         'shared': False}}
        self.run_scenario(expect_created)

    def test_format_networks_for_create(self):
        # format_for_create test setup
        # Basic test case
        net1_neutron = {'id': 'n1',
                        'project_id': 't1',
                        'name': 'n1_name',
                        'admin_state_up': True,
                        'rbac_entries': []}
        net1_expected = {'n1': {'id': 'n1',
                                'tenantId': 't1',
                                'name': 'n1_name',
                                'shared': False}}
        # Shared network
        shared_rbac = rbac_models.NetworkRBAC(**{'id': 1,
                                                 'project_id': 't2',
                                                 'target_tenant': '*',
                                                 'action': 'access_as_shared'})
        net2_neutron = {'id': 'n2',
                        'project_id': 't2',
                        'name': '',
                        'admin_state_up': True,
                        'rbac_entries': [shared_rbac]}
        net2_expected = {'n2': {'id': 'n2',
                                'tenantId': 't2',
                                'name': '',
                                'shared': True}}
        # Other RBAC
        other_rbac1 = rbac_models.NetworkRBAC(**{'id': 2,
                                                 'project_id': 't3',
                                                 'target_tenant': 't1',
                                                 'action': 'access_as_shared'})
        other_rbac2 = rbac_models.NetworkRBAC(**{'id': 3,
                                                 'project_id': 't3',
                                                 'target_tenant': 't2',
                                                 'action': 'access_as_shared'})
        net3_neutron = {'id': 'n3',
                        'project_id': 't3',
                        'name': 'n3_name',
                        'admin_state_up': True,
                        'rbac_entries': [other_rbac1, other_rbac2]}
        net3_expected = {'n3': {'id': 'n3',
                                'tenantId': 't3',
                                'name': 'n3_name',
                                'shared': False}}
        utils.create_networks([net1_neutron, net2_neutron, net3_neutron])
        test_cases = [(db_lib.get_networks('n1')[0], net1_expected),
                      (db_lib.get_networks('n2')[0], net2_expected),
                      (db_lib.get_networks('n3')[0], net3_expected)]
        self.verify_format_for_create(test_cases)

    def test_format_networks_for_delete(self):
        # format_for_delete test setup
        neutron_net_id = 'n4'
        expected_net = {'id': 'n4'}
        test_case = [(neutron_net_id, expected_net)]
        self.verify_format_for_delete(test_case)


class AristaSegmentTest(AristaResourcesTestBase):
    """Test cases for creation & deletion of arista segments"""

    def setUp(self):
        super(AristaSegmentTest, self).setUp()
        self.ar = resources.Segments(self.rpc)

    def delete_helper(self, segment_id):
        utils.delete_segment(segment_id)

    def test_segments_scenario(self):
        expect_created = {'s1': {'id': 's1',
                                 'networkId': 'n1',
                                 'segmentType': 'static',
                                 'segmentationId': 11,
                                 'type': 'vlan'},
                          's2': {'id': 's2',
                                 'networkId': 'n2',
                                 'segmentType': 'static',
                                 'segmentationId': 20001,
                                 'type': 'vxlan'},
                          's3': {'id': 's3',
                                 'networkId': 'n2',
                                 'segmentType': 'dynamic',
                                 'segmentationId': 21,
                                 'type': 'vlan'},
                          's4': {'id': 's4',
                                 'networkId': 'n2',
                                 'segmentType': 'dynamic',
                                 'segmentationId': 31,
                                 'type': 'vlan'},
                          's5': {'id': 's5',
                                 'networkId': 'HA network',
                                 'segmentType': 'static',
                                 'segmentationId': 33,
                                 'type': 'vlan'},
                          's6': {'id': 's6',
                                 'networkId': 'HA network',
                                 'segmentType': 'static',
                                 'segmentationId': 20010,
                                 'type': 'vxlan'},
                          's7': {'id': 's7',
                                 'networkId': 'HA network',
                                 'segmentationId': 700,
                                 'segmentType': 'dynamic',
                                 'type': 'vlan'},
                          's8': {'id': 's8',
                                 'networkId': 'HA network',
                                 'segmentationId': 800,
                                 'segmentType': 'dynamic',
                                 'type': 'vlan'}}
        self.run_scenario(expect_created)

    def test_format_segments_for_create(self):
        seg1_neutron = {'id': 's1',
                        'network_id': 'nid',
                        'is_dynamic': False,
                        'segmentation_id': 10001,
                        'network_type': 'vxlan',
                        'physical_network': None}
        seg1_expected = {'s1': {'id': 's1',
                                'type': 'vxlan',
                                'segmentationId': 10001,
                                'networkId': 'nid',
                                'segmentType': 'static'}}
        seg2_neutron = {'id': 's2',
                        'network_id': 'nid',
                        'is_dynamic': True,
                        'segmentation_id': 11,
                        'network_type': 'vlan',
                        'physical_network': 'default'}
        seg2_expected = {'s2': {'id': 's2',
                                'type': 'vlan',
                                'segmentationId': 11,
                                'networkId': 'nid',
                                'segmentType': 'dynamic'}}
        test_cases = [(seg1_neutron, seg1_expected),
                      (seg2_neutron, seg2_expected)]
        self.verify_format_for_create(test_cases)

    def test_format_segments_for_delete(self):
        neutron_seg_id = 's3'
        expected_seg = {'id': 's3'}
        test_case = [(neutron_seg_id, expected_seg)]
        self.verify_format_for_delete(test_case)


class AristaInstancesTestBase(AristaResourcesTestBase):

    def delete_helper(self, instance_id):
        utils.delete_ports_for_instance(instance_id)


class AristaDhcpTest(AristaInstancesTestBase):
    """Test cases for creation & deletion of arista dhcp instances"""

    def setUp(self):
        super(AristaDhcpTest, self).setUp()
        self.ar = resources.Dhcps(self.rpc)

    def test_dhcps_scenario(self):
        id_base = n_const.DEVICE_OWNER_DHCP + 'normal'
        expect_created = {'%s1' % id_base:
                          {'tenantId': 't1',
                           'hostId': 'host1',
                           'id': '%s1' % id_base},
                          '%s2' % id_base:
                          {'tenantId': 't2',
                           'hostId': 'host2',
                           'id': '%s2' % id_base}}
        self.run_scenario(expect_created)

    def test_dhcps_managed_physnets_scenario(self):
        cfg.CONF.set_override('managed_physnets', 'switch1', 'ml2_arista')
        id_base = n_const.DEVICE_OWNER_DHCP + 'normal'
        expect_created = {'%s2' % id_base:
                          {'tenantId': 't2',
                           'hostId': 'host2',
                           'id': '%s2' % id_base}}
        self.run_scenario(expect_created)


class AristaRouterTest(AristaInstancesTestBase):
    """Test cases for creation & deletion of arista routers"""

    def setUp(self):
        super(AristaRouterTest, self).setUp()
        self.ar = resources.Routers(self.rpc)

    def test_routers_scenario(self):
        id_base = n_const.DEVICE_OWNER_DVR_INTERFACE + 'normal'
        ha_id_base = n_const.DEVICE_OWNER_ROUTER_HA_INTF + 'normal'
        legacy_id_base = n_const.DEVICE_OWNER_ROUTER_INTF + 'normal'
        expect_created = {'%s1' % id_base:
                          {'tenantId': 't1',
                           'hostId': 'distributed',
                           'id': '%s1' % id_base},
                          '%s2' % id_base:
                          {'tenantId': 't2',
                           'hostId': 'distributed',
                           'id': '%s2' % id_base},
                          '%s' % ha_id_base:
                          {'tenantId': 'ha-router-project',
                           'hostId': 'HA Router',
                           'id': '%s' % ha_id_base},
                          '%s1' % legacy_id_base:
                          {'tenantId': 't1',
                           'hostId': 'host1',
                           'id': '%s1' % legacy_id_base},
                          '%s2' % legacy_id_base:
                          {'tenantId': 't2',
                           'hostId': 'host2',
                           'id': '%s2' % legacy_id_base}}
        self.run_scenario(expect_created)

    def test_routers_managed_physnets_scenario(self):
        cfg.CONF.set_override('managed_physnets', 'switch1', 'ml2_arista')
        id_base = n_const.DEVICE_OWNER_DVR_INTERFACE + 'normal'
        legacy_id_base = n_const.DEVICE_OWNER_ROUTER_INTF + 'normal'
        ha_id_base = n_const.DEVICE_OWNER_ROUTER_HA_INTF + 'normal'
        expect_created = {'%s2' % id_base:
                          {'tenantId': 't2',
                           'hostId': 'distributed',
                           'id': '%s2' % id_base},
                          '%s2' % legacy_id_base:
                          {'id': '%s2' % legacy_id_base,
                           'tenantId': 't2',
                           'hostId': 'host2'},
                          '%s' % ha_id_base:
                          {'tenantId': 'ha-router-project',
                           'hostId': 'HA Router',
                           'id': '%s' % ha_id_base}}
        self.run_scenario(expect_created)


class AristaVmTest(AristaInstancesTestBase):
    """Test cases for creation & deletion of arista vms"""

    def setUp(self):
        super(AristaVmTest, self).setUp()
        self.ar = resources.Vms(self.rpc)

    def test_vms_scenario(self):
        id_base = n_const.DEVICE_OWNER_COMPUTE_PREFIX + 'normal'
        expect_created = {'%s1' % id_base:
                          {'tenantId': 't1',
                           'hostId': 'host1',
                           'id': '%s1' % id_base},
                          '%s2' % id_base:
                          {'tenantId': 't2',
                           'hostId': 'host2',
                           'id': '%s2' % id_base}}
        self.run_scenario(expect_created)

    def test_vms_managed_physnets_scenario(self):
        cfg.CONF.set_override('managed_physnets', 'switch1', 'ml2_arista')
        id_base = n_const.DEVICE_OWNER_COMPUTE_PREFIX + 'normal'
        expect_created = {'%s2' % id_base:
                          {'tenantId': 't2',
                           'hostId': 'host2',
                           'id': '%s2' % id_base}}
        self.run_scenario(expect_created)


class AristaBaremetalTest(AristaInstancesTestBase):
    """Test cases for creation & deletion of arista baremetal instances"""

    def setUp(self):
        super(AristaBaremetalTest, self).setUp()
        self.ar = resources.Baremetals(self.rpc)

    def test_baremetals_scenario(self):
        id_base = n_const.DEVICE_OWNER_COMPUTE_PREFIX + 'baremetal'
        legacy_id_base = n_const.DEVICE_OWNER_BAREMETAL_PREFIX + 'baremetal'
        expect_created = {'%s1' % id_base:
                          {'tenantId': 't1',
                           'hostId': 'host1',
                           'id': '%s1' % id_base},
                          '%s2' % id_base:
                          {'tenantId': 't2',
                           'hostId': 'host2',
                           'id': '%s2' % id_base},
                          '%s1' % legacy_id_base:
                          {'tenantId': 't1',
                           'hostId': 'host1',
                           'id': '%s1' % legacy_id_base},
                          '%s2' % legacy_id_base:
                          {'tenantId': 't2',
                           'hostId': 'host2',
                           'id': '%s2' % legacy_id_base}}
        self.run_scenario(expect_created)

    def test_baremetals_managed_physnets_scenario(self):
        cfg.CONF.set_override('managed_physnets', 'switch1', 'ml2_arista')
        id_base = n_const.DEVICE_OWNER_COMPUTE_PREFIX + 'baremetal'
        legacy_id_base = n_const.DEVICE_OWNER_BAREMETAL_PREFIX + 'baremetal'
        expect_created = {'%s2' % id_base:
                          {'tenantId': 't2',
                           'hostId': 'host2',
                           'id': '%s2' % id_base},
                          '%s2' % legacy_id_base:
                          {'tenantId': 't2',
                           'hostId': 'host2',
                           'id': '%s2' % legacy_id_base}}
        self.run_scenario(expect_created)


class AristaPortTestBase(AristaResourcesTestBase):

    def delete_helper(self, port_id):
        utils.delete_port(port_id)


class AristaDhcpPortTest(AristaPortTestBase):
    """Test cases for creation & deletion of arista dhcp ports"""

    def setUp(self):
        super(AristaDhcpPortTest, self).setUp()
        self.ar = resources.DhcpPorts(self.rpc)

    def test_dhcp_ports_scenario(self):
        id_base = n_const.DEVICE_OWNER_DHCP + 'normal'
        expect_created = {'p1': {'id': 'p1',
                                 'portName': 'regular_port',
                                 'tenantId': 't1',
                                 'instanceType': 'dhcp',
                                 'instanceId': '%s1' % id_base,
                                 'networkId': 'n1',
                                 'vlanType': 'allowed'},
                          'p2': {'id': 'p2',
                                 'portName': 'hpb_port',
                                 'tenantId': 't2',
                                 'instanceType': 'dhcp',
                                 'instanceId': '%s2' % id_base,
                                 'networkId': 'n2',
                                 'vlanType': 'allowed'}}
        self.run_scenario(expect_created)

    def test_dhcp_ports_managed_physnets_scenario(self):
        cfg.CONF.set_override('managed_physnets', 'switch1', 'ml2_arista')
        id_base = n_const.DEVICE_OWNER_DHCP + 'normal'
        expect_created = {'p2': {'id': 'p2',
                                 'portName': 'hpb_port',
                                 'tenantId': 't2',
                                 'instanceType': 'dhcp',
                                 'instanceId': '%s2' % id_base,
                                 'networkId': 'n2',
                                 'vlanType': 'allowed'}}
        self.run_scenario(expect_created)


class AristaRouterPortTest(AristaPortTestBase):
    """Test cases for creation & deletion of arista router ports"""

    def setUp(self):
        super(AristaRouterPortTest, self).setUp()
        self.ar = resources.RouterPorts(self.rpc)

    def test_router_ports_scenario(self):
        id_base = n_const.DEVICE_OWNER_DVR_INTERFACE + 'normal'
        legacy_id_base = n_const.DEVICE_OWNER_ROUTER_INTF + 'normal'
        ha_id_base = n_const.DEVICE_OWNER_ROUTER_HA_INTF + 'normal'
        expect_created = {'p3': {'id': 'p3',
                                 'portName': 'regular_port',
                                 'tenantId': 't1',
                                 'instanceType': 'router',
                                 'instanceId': '%s1' % id_base,
                                 'networkId': 'n1',
                                 'vlanType': 'allowed'},
                          'p4': {'id': 'p4',
                                 'portName': 'hpb_port',
                                 'tenantId': 't2',
                                 'instanceType': 'router',
                                 'instanceId': '%s2' % id_base,
                                 'networkId': 'n2',
                                 'vlanType': 'allowed'},
                          'uuid-ha-1': {
                              'id': 'uuid-ha-1',
                              'portName': 'regular_port',
                              'tenantId': 'ha-router-project',
                              'instanceType': 'router',
                              'instanceId': '%s' % ha_id_base,
                              'networkId': 'HA network',
                              'vlanType': 'allowed'},
                          'uuid-ha-2': {
                              'id': 'uuid-ha-2',
                              'portName': 'regular_port',
                              'tenantId': 'ha-router-project',
                              'instanceType': 'router',
                              'instanceId': '%s' % ha_id_base,
                              'networkId': 'HA network',
                              'vlanType': 'allowed'},
                          'uuid-hpb-ha-1': {
                              'id': 'uuid-hpb-ha-1',
                              'portName': 'ha_router_hpb_port',
                              'tenantId': 'ha-router-project',
                              'instanceType': 'router',
                              'instanceId': '%s' % ha_id_base,
                              'networkId': 'HA network',
                              'vlanType': 'allowed'},
                          'uuid-hpb-ha-2': {
                              'id': 'uuid-hpb-ha-2',
                              'portName': u'ha_router_hpb_port',
                              'tenantId': 'ha-router-project',
                              'instanceType': 'router',
                              'networkId': u'HA network',
                              'instanceId': '%s' % ha_id_base,
                              'vlanType': 'allowed'},
                          'p15': {'id': 'p15',
                                  'portName': u'regular_port',
                                  'networkId': 'n1',
                                  'instanceId': '%s1' % legacy_id_base,
                                  'instanceType': 'router',
                                  'tenantId': 't1',
                                  'vlanType': 'allowed'},
                          'p16': {'id': 'p16',
                                  'portName': 'hpb_port',
                                  'tenantId': 't2',
                                  'instanceId': '%s2' % legacy_id_base,
                                  'instanceType': 'router',
                                  'networkId': 'n2',
                                  'vlanType': 'allowed'}}
        self.run_scenario(expect_created)

    def test_router_ports_managed_physnets_scenario(self):
        cfg.CONF.set_override('managed_physnets', 'switch1', 'ml2_arista')
        id_base = n_const.DEVICE_OWNER_DVR_INTERFACE + 'normal'
        legacy_id_base = n_const.DEVICE_OWNER_ROUTER_INTF + 'normal'
        ha_id_base = n_const.DEVICE_OWNER_ROUTER_HA_INTF + 'normal'
        expect_created = {'p4': {'id': 'p4',
                                 'portName': 'hpb_port',
                                 'tenantId': 't2',
                                 'instanceType': 'router',
                                 'instanceId': '%s2' % id_base,
                                 'networkId': 'n2',
                                 'vlanType': 'allowed'},
                          'uuid-hpb-ha-1': {
                              'id': 'uuid-hpb-ha-1',
                              'portName': 'ha_router_hpb_port',
                              'tenantId': 'ha-router-project',
                              'instanceType': 'router',
                              'instanceId': '%s' % ha_id_base,
                              'networkId': 'HA network',
                              'vlanType': 'allowed'},
                          'p16': {'id': 'p16',
                                  'portName': 'hpb_port',
                                  'tenantId': 't2',
                                  'instanceType': 'router',
                                  'instanceId': '%s2' % legacy_id_base,
                                  'networkId': 'n2',
                                  'vlanType': 'allowed'}}
        self.run_scenario(expect_created)


class AristaVmPortTest(AristaPortTestBase):
    """Test cases for creation & deletion of arista vm ports"""

    def setUp(self):
        super(AristaVmPortTest, self).setUp()
        self.ar = resources.VmPorts(self.rpc)

    def test_vm_ports_scenario(self):
        id_base = n_const.DEVICE_OWNER_COMPUTE_PREFIX + 'normal'
        expect_created = {'p5': {'id': 'p5',
                                 'portName': 'regular_port',
                                 'tenantId': 't1',
                                 'instanceType': 'vm',
                                 'instanceId': '%s1' % id_base,
                                 'networkId': 'n1',
                                 'vlanType': 'allowed'},
                          'p6': {'id': 'p6',
                                 'portName': 'hpb_port',
                                 'tenantId': 't2',
                                 'instanceType': 'vm',
                                 'instanceId': '%s2' % id_base,
                                 'networkId': 'n2',
                                 'vlanType': 'allowed'},
                          'p7': {'id': 'p7',
                                 'portName': 'trunk_subport',
                                 'tenantId': 't1',
                                 'instanceType': 'vm',
                                 'instanceId': '%s1' % id_base,
                                 'networkId': 'n1',
                                 'vlanType': 'allowed'}}
        self.run_scenario(expect_created)

    def test_vm_ports_managed_physnets_scenario(self):
        cfg.CONF.set_override('managed_physnets', 'switch1', 'ml2_arista')
        id_base = n_const.DEVICE_OWNER_COMPUTE_PREFIX + 'normal'
        expect_created = {'p6': {'id': 'p6',
                                 'portName': 'hpb_port',
                                 'tenantId': 't2',
                                 'instanceType': 'vm',
                                 'instanceId': '%s2' % id_base,
                                 'networkId': 'n2',
                                 'vlanType': 'allowed'}}
        self.run_scenario(expect_created)


class AristaBaremetalPortTest(AristaPortTestBase):
    """Test cases for creation & deletion of arista baremetal ports"""

    def setUp(self):
        super(AristaBaremetalPortTest, self).setUp()
        self.ar = resources.BaremetalPorts(self.rpc)

    def test_baremetal_ports_scenario(self):
        id_base = n_const.DEVICE_OWNER_COMPUTE_PREFIX + 'baremetal'
        legacy_id_base = n_const.DEVICE_OWNER_BAREMETAL_PREFIX + 'baremetal'
        expect_created = {'p8': {'id': 'p8',
                                 'portName': 'regular_port',
                                 'tenantId': 't1',
                                 'instanceType': 'baremetal',
                                 'instanceId': '%s1' % id_base,
                                 'networkId': 'n1',
                                 'vlanType': 'native'},
                          'p9': {'id': 'p9',
                                 'portName': 'hpb_port',
                                 'tenantId': 't2',
                                 'instanceType': 'baremetal',
                                 'instanceId': '%s2' % id_base,
                                 'networkId': 'n2',
                                 'vlanType': 'native'},
                          'p10': {'id': 'p10',
                                  'portName': 'trunk_subport',
                                  'tenantId': 't1',
                                  'instanceType': 'baremetal',
                                  'instanceId': '%s1' % id_base,
                                  'networkId': 'n1',
                                  'vlanType': 'allowed'},
                          'p11': {'id': 'p11',
                                  'portName': 'regular_port',
                                  'tenantId': 't1',
                                  'instanceType': 'baremetal',
                                  'instanceId': '%s1' % legacy_id_base,
                                  'networkId': 'n1',
                                  'vlanType': 'native'},
                          'p12': {'id': 'p12',
                                  'portName': 'hpb_port',
                                  'tenantId': 't2',
                                  'instanceType': 'baremetal',
                                  'instanceId': '%s2' % legacy_id_base,
                                  'networkId': 'n2',
                                  'vlanType': 'native'}}
        self.run_scenario(expect_created)

    def test_baremetal_ports_managed_physnets_scenario(self):
        cfg.CONF.set_override('managed_physnets', 'switch1', 'ml2_arista')
        id_base = n_const.DEVICE_OWNER_COMPUTE_PREFIX + 'baremetal'
        legacy_id_base = n_const.DEVICE_OWNER_BAREMETAL_PREFIX + 'baremetal'
        expect_created = {'p9': {'id': 'p9',
                                 'portName': 'hpb_port',
                                 'tenantId': 't2',
                                 'instanceType': 'baremetal',
                                 'instanceId': '%s2' % id_base,
                                 'networkId': 'n2',
                                 'vlanType': 'native'},
                          'p12': {'id': 'p12',
                                  'portName': 'hpb_port',
                                  'tenantId': 't2',
                                  'instanceType': 'baremetal',
                                  'instanceId': '%s2' % legacy_id_base,
                                  'networkId': 'n2',
                                  'vlanType': 'native'}}
        self.run_scenario(expect_created)


class AristaPortBindingTest(AristaResourcesTestBase):
    """Test cases for creation & deletion of arista port bindings"""

    def setUp(self):
        super(AristaPortBindingTest, self).setUp()
        self.ar = resources.PortBindings(self.rpc)

    def delete_helper(self, binding_key):
        if type(binding_key[1]) == tuple:
            utils.remove_switch_binding(binding_key[0],
                                        *binding_key[1])
        else:
            utils.delete_port_binding(*binding_key)

    def test_port_binding_scenario(self):
        expect_created = {
            # DHCP ports
            ('p1', 'host1'):
            {'portId': 'p1',
             'hostBinding': [{'host': 'host1',
                              'segment': [{'id': 's1'}]}],
             'switchBinding': []},
            ('p2', 'host2'):
            {'portId': 'p2',
             'hostBinding': [{'host': 'host2',
                              'segment': [{'id': 's2'},
                                          {'id': 's3'}]}],
             'switchBinding': []},
            # DVR ports
            ('p3', 'host1'):
            {'portId': 'p3',
             'hostBinding': [{'host': 'host1',
                              'segment': [{'id': 's1'}]}],
             'switchBinding': []},
            ('p3', 'host2'):
            {'portId': 'p3',
             'hostBinding': [{'host': 'host2',
                              'segment': [{'id': 's1'}]}],
             'switchBinding': []},
            ('p4', 'host1'):
            {'portId': 'p4',
             'hostBinding': [{'host': 'host1',
                              'segment': [{'id': 's2'},
                                          {'id': 's3'}]}],
             'switchBinding': []},
            ('p4', 'host2'):
            {'portId': 'p4',
             'hostBinding': [{'host': 'host2',
                              'segment': [{'id': 's2'},
                                          {'id': 's4'}]}],
             'switchBinding': []},
            # VM ports
            ('p5', 'host1'):
            {'portId': 'p5',
             'hostBinding': [{'host': 'host1',
                              'segment': [{'id': 's1'}]}],
             'switchBinding': []},
            ('p6', 'host2'):
            {'portId': 'p6',
             'hostBinding': [{'host': 'host2',
                              'segment': [{'id': 's2'},
                                          {'id': 's3'}]}],
             'switchBinding': []},
            ('p7', 'host1'):
            {'portId': 'p7',
             'hostBinding': [{'host': 'host1',
                              'segment': [{'id': 's1'}]}],
             'switchBinding': []},
            # Baremetal ports
            ('p8', ('00:11:22:33:44:55', 'Ethernet1')):
            {'portId': 'p8',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet1',
                                'switch': '00:11:22:33:44:55',
                                'segment': [{'id': 's1'}]}]},
            ('p8', ('00:11:22:33:44:55', 'Ethernet2')):
            {'portId': 'p8',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet2',
                                'segment': [{'id': 's1'}],
                                'switch': '00:11:22:33:44:55'}]},
            ('p8', ('55:44:33:22:11:00', 'Ethernet1/1')):
            {'portId': 'p8',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet1/1',
                                'segment': [{'id': 's1'}],
                                'switch': '55:44:33:22:11:00'}]},
            ('p8', ('55:44:33:22:11:00', 'Ethernet1/2')):
            {'portId': 'p8',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet1/2',
                                'segment': [{'id': 's1'}],
                                'switch': '55:44:33:22:11:00'}]},
            ('p9', ('00:11:22:33:44:55', 'Ethernet1')):
            {'portId': 'p9',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1',
                                'switch': '00:11:22:33:44:55',
                                'segment': [{'id': 's2'}, {'id': 's3'}]}]},
            ('p9', ('00:11:22:33:44:55', 'Ethernet2')):
            {'portId': 'p9',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet2',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '00:11:22:33:44:55'}]},
            ('p9', ('55:44:33:22:11:00', 'Ethernet1/1')):
            {'portId': 'p9',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1/1',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '55:44:33:22:11:00'}]},
            ('p9', ('55:44:33:22:11:00', 'Ethernet1/2')):
            {'portId': 'p9',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1/2',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '55:44:33:22:11:00'}]},
            ('p10', ('00:11:22:33:44:55', 'Ethernet1')):
            {'portId': 'p10',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet1',
                                'switch': '00:11:22:33:44:55',
                                'segment': [{'id': 's1'}]}]},
            ('p10', ('00:11:22:33:44:55', 'Ethernet2')):
            {'portId': 'p10',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet2',
                                'segment': [{'id': 's1'}],
                                'switch': '00:11:22:33:44:55'}]},
            ('p10', ('55:44:33:22:11:00', 'Ethernet1/1')):
            {'portId': 'p10',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet1/1',
                                'segment': [{'id': 's1'}],
                                'switch': '55:44:33:22:11:00'}]},
            ('p10', ('55:44:33:22:11:00', 'Ethernet1/2')):
            {'portId': 'p10',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet1/2',
                                'segment': [{'id': 's1'}],
                                'switch': '55:44:33:22:11:00'}]},
            # Legacy baremetal ports
            ('p11', ('00:11:22:33:44:55', 'Ethernet1')):
            {'portId': 'p11',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet1',
                                'switch': '00:11:22:33:44:55',
                                'segment': [{'id': 's1'}]}]},
            ('p11', ('00:11:22:33:44:55', 'Ethernet2')):
            {'portId': 'p11',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet2',
                                'segment': [{'id': 's1'}],
                                'switch': '00:11:22:33:44:55'}]},
            ('p11', ('55:44:33:22:11:00', 'Ethernet1/1')):
            {'portId': 'p11',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet1/1',
                                'segment': [{'id': 's1'}],
                                'switch': '55:44:33:22:11:00'}]},
            ('p11', ('55:44:33:22:11:00', 'Ethernet1/2')):
            {'portId': 'p11',
             'hostBinding': [],
             'switchBinding': [{'host': 'host1',
                                'interface': 'Ethernet1/2',
                                'segment': [{'id': 's1'}],
                                'switch': '55:44:33:22:11:00'}]},
            ('p12', ('00:11:22:33:44:55', 'Ethernet1')):
            {'portId': 'p12',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1',
                                'switch': '00:11:22:33:44:55',
                                'segment': [{'id': 's2'}, {'id': 's3'}]}]},
            ('p12', ('00:11:22:33:44:55', 'Ethernet2')):
            {'portId': 'p12',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet2',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '00:11:22:33:44:55'}]},
            ('p12', ('55:44:33:22:11:00', 'Ethernet1/1')):
            {'portId': 'p12',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1/1',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '55:44:33:22:11:00'}]},
            ('p12', ('55:44:33:22:11:00', 'Ethernet1/2')):
            {'portId': 'p12',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1/2',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '55:44:33:22:11:00'}]},
            # HA network port
            ('uuid-ha-1', ('host1')):
            {'portId': 'uuid-ha-1',
             'switchBinding': [],
             'hostBinding': [{'segment': [{'id': 's5'}], 'host': 'host1'}]},
            ('uuid-ha-2', ('host2')):
            {'portId': 'uuid-ha-2',
             'switchBinding': [],
             'hostBinding': [{'segment': [{'id': 's5'}], 'host': 'host2'}]},
            ('uuid-hpb-ha-1', ('host1')):
            {'portId': 'uuid-hpb-ha-1',
             'switchBinding': [],
             'hostBinding': [{'segment': [{'id': 's6'}, {'id': 's7'}],
                              'host': 'host1'}]},
            ('uuid-hpb-ha-2', ('host2')):
            {'portId': 'uuid-hpb-ha-2',
             'switchBinding': [],
             'hostBinding': [{'segment': [{'id': 's6'}, {'id': 's8'}],
                              'host': 'host2'}]},
            # Legacy router port
            ('p15', ('host1')):
            {'portId': 'p15',
             'switchBinding': [],
             'hostBinding': [{'segment': [{'id': 's1'}], 'host': 'host1'}]},
            ('p16', ('host2')):
            {'portId': 'p16',
             'switchBinding': [],
             'hostBinding': [{'segment': [{'id': 's2'}, {'id': 's3'}],
                              'host': 'host2'}]}}
        self.run_scenario(expect_created)

    def test_port_binding_managed_physnets_scenario(self):
        cfg.CONF.set_override('managed_physnets', 'switch1', 'ml2_arista')
        expect_created = {
            # DHCP ports
            ('p2', 'host2'):
            {'portId': 'p2',
             'hostBinding': [{'host': 'host2',
                              'segment': [{'id': 's2'},
                                          {'id': 's3'}]}],
             'switchBinding': []},
            # DVR ports
            ('p4', 'host1'):
            {'portId': 'p4',
             'hostBinding': [{'host': 'host1',
                              'segment': [{'id': 's2'},
                                          {'id': 's3'}]}],
             'switchBinding': []},
            # VM ports
            ('p6', 'host2'):
            {'portId': 'p6',
             'hostBinding': [{'host': 'host2',
                              'segment': [{'id': 's2'},
                                          {'id': 's3'}]}],
             'switchBinding': []},
            # Baremetal ports
            ('p9', ('00:11:22:33:44:55', 'Ethernet1')):
            {'portId': 'p9',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1',
                                'switch': '00:11:22:33:44:55',
                                'segment': [{'id': 's2'}, {'id': 's3'}]}]},
            ('p9', ('00:11:22:33:44:55', 'Ethernet2')):
            {'portId': 'p9',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet2',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '00:11:22:33:44:55'}]},
            ('p9', ('55:44:33:22:11:00', 'Ethernet1/1')):
            {'portId': 'p9',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1/1',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '55:44:33:22:11:00'}]},
            ('p9', ('55:44:33:22:11:00', 'Ethernet1/2')):
            {'portId': 'p9',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1/2',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '55:44:33:22:11:00'}]},
            # Legacy baremetal ports
            ('p12', ('00:11:22:33:44:55', 'Ethernet1')):
            {'portId': 'p12',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1',
                                'switch': '00:11:22:33:44:55',
                                'segment': [{'id': 's2'}, {'id': 's3'}]}]},
            ('p12', ('00:11:22:33:44:55', 'Ethernet2')):
            {'portId': 'p12',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet2',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '00:11:22:33:44:55'}]},
            ('p12', ('55:44:33:22:11:00', 'Ethernet1/1')):
            {'portId': 'p12',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1/1',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '55:44:33:22:11:00'}]},
            ('p12', ('55:44:33:22:11:00', 'Ethernet1/2')):
            {'portId': 'p12',
             'hostBinding': [],
             'switchBinding': [{'host': 'host2',
                                'interface': 'Ethernet1/2',
                                'segment': [{'id': 's2'}, {'id': 's3'}],
                                'switch': '55:44:33:22:11:00'}]},
            # Legacy router
            ('p16', ('host2')):
            {'portId': 'p16',
             'switchBinding': [],
             'hostBinding': [{'segment': [{'id': 's2'},
                                          {'id': 's3'}], 'host': 'host2'}]},
            ('uuid-hpb-ha-1', ('host1')):
            {'portId': 'uuid-hpb-ha-1',
             'switchBinding': [],
             'hostBinding': [{'segment': [{'id': 's6'}, {'id': 's7'}],
                              'host': 'host1'}]}}
        self.run_scenario(expect_created)



# Copyright (c) 2018 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re

from neutron_lib import constants as n_const
from oslo_config import cfg
from oslo_log import log as logging

from networking_arista._i18n import _LI
from networking_arista.common import api
from networking_arista.common import constants as a_const
from networking_arista.common import exceptions as arista_exc
from networking_arista.common import utils

LOG = logging.getLogger(__name__)


class AristaSecurityGroupSwitchHelper(object):
    """Helper class for applying baremetal security groups on Arista switches

    This helper class contains methods for adding and removing security
    groups, security group rules and security group port bindings to and from
    Arista switches.
    """

    def initialize_switch_endpoints(self):
        """Initialize endpoints for switch communication"""
        self._switches = {}
        self._port_group_info = {}
        self._validate_config()
        for s in cfg.CONF.ml2_arista.switch_info:
            switch_ip, switch_user, switch_pass = s.split(":")
            if switch_pass == "''":
                switch_pass = ''
            self._switches[switch_ip] = api.EAPIClient(
                switch_ip,
                switch_user,
                switch_pass,
                verify=False,
                timeout=cfg.CONF.ml2_arista.conn_timeout)

    def _validate_config(self):
        """Ensure at least one switch is configured"""
        if len(cfg.CONF.ml2_arista.get('switch_info')) < 1:
            msg = _('Required option - when "sec_group_support" is enabled, '
                    'at least one switch must be specified ')
            LOG.exception(msg)
            raise arista_exc.AristaConfigError(msg=msg)

    def _run_openstack_sg_cmds(self, commands, switch):
        """Execute/sends a CAPI (Command API) command to EOS.

        In this method, list of commands is appended with prefix and
        postfix commands - to make is understandble by EOS.

        :param commands : List of command to be executed on EOS.
        :param switch: Endpoint on the Arista switch to be configured
        """
        if len(commands) == 0:
            return []
        command_start = ['enable', 'configure']
        command_end = ['exit']
        full_command = command_start + commands + command_end
        return self._run_eos_cmds(full_command, switch)

    def _run_eos_cmds(self, commands, switch):
        """Execute/sends a CAPI (Command API) command to EOS.

        This method is useful for running show commands that require no
        prefix or postfix commands.

        :param commands : List of commands to be executed on EOS.
        :param switch: Endpoint on the Arista switch to be configured
        """
        LOG.info(_LI('Executing command on Arista EOS: %s'), commands)

        try:
            # this returns array of return values for every command in
            # commands list
            ret = switch.execute(commands)
            LOG.info(_LI('Results of execution on Arista EOS: %s'), ret)
            return ret
        except Exception:
            msg = (_('Error occurred while trying to execute '
                     'commands %(cmd)s on EOS %(host)s') %
                   {'cmd': commands, 'host': switch})
            LOG.exception(msg)

    def run_cmds_on_all_switches(self, cmds):
        """Runs all cmds on all configured switches

        This helper is used for ACL and rule creation/deletion as ACLs
        and rules must exist on all switches.
        """
        for switch in self._switches.values():
            self._run_openstack_sg_cmds(cmds, switch)

    def run_per_switch_cmds(self, switch_cmds):
        """Applies cmds to appropriate switches

        This takes in a switch->cmds mapping and runs only the set of cmds
        specified for a switch on that switch. This helper is used for
        applying/removing ACLs to/from interfaces as this config will vary
        from switch to switch.
        """
        for switch_ip, cmds in switch_cmds.items():
            switch = self._switches[switch_ip]
            self._run_openstack_sg_cmds(cmds, switch)

    @staticmethod
    def _acl_name(name, direction):
        """Generate an arista specific name for this ACL.

        Use a unique name so that OpenStack created ACLs
        can be distinguishged from the user created ACLs
        on Arista HW.
        """
        direction = direction.upper()
        return 'SG' + '-' + direction + '-' + name

    def _get_switches(self, profile):
        """Get set of switches referenced in a port binding profile"""
        switchports = self._get_switchports(profile)
        switches = set([switchport[0] for switchport in switchports])
        return switches

    @staticmethod
    def _get_switchports(profile):
        """Return list of (switch_ip, interface) tuples from local_link_info"""
        switchports = []
        if profile.get('local_link_information'):
            for link in profile['local_link_information']:
                switch = link['switch_info']
                interface = link['port_id']
                switchports.append((switch, interface))
        return switchports

    def _update_port_group_info(self, switches=None):
        """Refresh data on switch interfaces' port group membership"""
        if switches is None:
            switches = self._switches.keys()
        for switch_ip in switches:
            client = self._switches[switch_ip]
            ret = self._run_eos_cmds(['show interfaces'], client)
            if not ret or len(ret) == 0:
                LOG.warning("Unable to retrieve interface info for %s",
                            switch_ip)
                continue
            intf_info = ret[0]
            self._port_group_info[switch_ip] = intf_info.get('interfaces', {})

    def _get_port_for_acl(self, port_id, switch):
        """Gets interface name for ACLs

        Finds the Port-Channel name if port_id is in a Port-Channel, otherwise
        ACLs are applied to Ethernet interface.

        :param port_id: Name of port from ironic db
        :param server: Server endpoint on the Arista switch to be configured
        """
        all_intf_info = self._port_group_info.get(switch, {})
        intf_info = all_intf_info.get(port_id, {})
        member_info = intf_info.get('interfaceMembership', '')
        port_group_info = re.search('Member of (?P<port_group>\S+)',
                                    member_info)
        if port_group_info:
            port_id = port_group_info.group('port_group')
        return port_id

    @staticmethod
    def _supported_rule(protocol, ethertype):
        """Checks that the rule is an IPv4 rule of a supported protocol"""
        if not protocol or protocol not in utils.SUPPORTED_SG_PROTOCOLS:
            return False

        if ethertype != n_const.IPv4:
            return False

        return True

    def _format_rule(self, protocol, cidr, min_port, max_port, direction):
        """Get EOS formatted rule"""
        if cidr is None:
            cidr = 'any'

        if direction == n_const.INGRESS_DIRECTION:
            dst_ip = 'any'
            src_ip = cidr
        elif direction == n_const.EGRESS_DIRECTION:
            dst_ip = cidr
            src_ip = 'any'

        if protocol == n_const.PROTO_NAME_ICMP:
            rule = "permit icmp %s %s" % (src_ip, dst_ip)
            if min_port:
                rule += " %s" % (min_port)
                if max_port:
                    rule += " %s" % (max_port)
        else:
            rule = "permit %s %s %s" % (protocol, src_ip, dst_ip)
            if min_port and max_port:
                rule += " range %s %s" % (min_port, max_port)
            elif min_port and not max_port:
                rule += " eq %s" % min_port
        return rule

    def _format_rules_for_eos(self, rules):
        """Format list of rules for EOS and sort into ingress/egress rules"""
        in_rules = []
        eg_rules = []
        for rule in rules:
            protocol = rule.get('protocol')
            cidr = rule.get('remote_ip_prefix', 'any')
            min_port = rule.get('port_range_min')
            max_port = rule.get('port_range_max')
            direction = rule.get('direction')
            ethertype = rule.get('ethertype')
            if not self._supported_rule(protocol, ethertype):
                continue
            formatted_rule = self._format_rule(protocol, cidr, min_port,
                                               max_port, direction)
            if rule['direction'] == n_const.INGRESS_DIRECTION:
                in_rules.append(formatted_rule)
            elif rule['direction'] == n_const.EGRESS_DIRECTION:
                eg_rules.append(formatted_rule)
        return in_rules, eg_rules

    def get_create_security_group_commands(self, sg_id, sg_rules):
        """Commands for creating ACL"""
        cmds = []
        in_rules, eg_rules = self._format_rules_for_eos(sg_rules)
        cmds.append("ip access-list %s" %
                    self._acl_name(sg_id, n_const.INGRESS_DIRECTION))
        cmds.append("no 1-$")
        for in_rule in in_rules:
            cmds.append(in_rule)
        cmds.append("exit")
        cmds.append("ip access-list %s" %
                    self._acl_name(sg_id, n_const.EGRESS_DIRECTION))
        cmds.append("no 1-$")
        for eg_rule in eg_rules:
            cmds.append(eg_rule)
        cmds.append("exit")
        return cmds

    def get_delete_security_group_commands(self, sg_id):
        """Commands for deleting ACL"""
        cmds = []
        cmds.append("no ip access-list %s" %
                    self._acl_name(sg_id, n_const.INGRESS_DIRECTION))
        cmds.append("no ip access-list %s" %
                    self._acl_name(sg_id, n_const.EGRESS_DIRECTION))
        return cmds

    def _get_rule_cmds(self, sg_id, sg_rule, delete=False):
        """Helper for getting add/delete ACL rule commands"""
        rule_prefix = ""
        if delete:
            rule_prefix = "no "
        in_rules, eg_rules = self._format_rules_for_eos([sg_rule])
        cmds = []
        if in_rules:
            cmds.append("ip access-list %s" %
                        self._acl_name(sg_id, n_const.INGRESS_DIRECTION))
            for in_rule in in_rules:
                cmds.append(rule_prefix + in_rule)
            cmds.append("exit")
        if eg_rules:
            cmds.append("ip access-list %s" %
                        self._acl_name(sg_id, n_const.EGRESS_DIRECTION))
            for eg_rule in eg_rules:
                cmds.append(rule_prefix + eg_rule)
            cmds.append("exit")
        return cmds

    def get_create_security_group_rule_commands(self, sg_id, sg_rule):
        """Commands for adding rule to ACL"""
        return self._get_rule_cmds(sg_id, sg_rule)

    def get_delete_security_group_rule_commands(self, sg_id, sg_rule):
        """Commands for removing rule from ACLS"""
        return self._get_rule_cmds(sg_id, sg_rule, delete=True)

    def _get_interface_commands(self, sg_id, profile, delete=False):
        """Helper for getting interface ACL apply/remove commands"""
        rule_prefix = ""
        if delete:
            rule_prefix = "no "
        switch_cmds = {}
        switchports = self._get_switchports(profile)
        for switch_ip, intf in switchports:
            cmds = []
            intf_id = self._get_port_for_acl(intf, switch_ip)
            cmds.append("interface %s" % intf_id)
            name = self._acl_name(sg_id, n_const.INGRESS_DIRECTION)
            cmds.append(rule_prefix + "ip access-group %s %s" %
                        (name, a_const.INGRESS_DIRECTION))
            name = self._acl_name(sg_id, n_const.EGRESS_DIRECTION)
            cmds.append(rule_prefix + "ip access-group %s %s" %
                        (name, a_const.EGRESS_DIRECTION))
            cmds.append("exit")
            if switch_ip not in switch_cmds.keys():
                switch_cmds[switch_ip] = []
            switch_cmds[switch_ip].extend(cmds)
        return switch_cmds

    def get_apply_security_group_commands(self, sg_id, profile):
        """Commands for applying ACL to interface"""
        return self._get_interface_commands(sg_id, profile)

    def get_remove_security_group_commands(self, sg_id, profile):
        """Commands for removing ACL from interface"""
        return self._get_interface_commands(sg_id, profile, delete=True)



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron_lib.plugins import constants as plugin_constants
from neutron_lib.plugins import directory
from oslo_utils import importutils

from neutron.tests.unit import testlib_api

from networking_arista.common import db_lib
from networking_arista.tests.unit import utils


class DbLibTest(testlib_api.SqlTestCase):
    """Test cases for database helper functions."""

    def setUp(self):
        super(DbLibTest, self).setUp()
        plugin_klass = importutils.import_class(
            "neutron.db.db_base_plugin_v2.NeutronDbPluginV2")
        directory.add_plugin(plugin_constants.CORE, plugin_klass())

    def test_get_tenants_empty(self):
        tenants = db_lib.get_tenants()
        self.assertEqual(tenants, [])

    def test_get_tenants_from_networks(self):
        tenant_1_id = 't1'
        tenant_2_id = 't2'
        utils.create_networks([{'id': 'n1',
                                'project_id': tenant_1_id},
                               {'id': 'n2',
                                'project_id': tenant_2_id}])
        tenants = db_lib.get_tenants()
        expected_tenants = [{'project_id': tenant_1_id},
                            {'project_id': tenant_2_id}]
        self.assertItemsEqual(tenants, expected_tenants)

    def test_get_network_with_no_tenant(self):
        tenant_1_id = 't1'
        tenant_2_id = ''
        utils.create_networks([{'id': 'n1',
                                'project_id': tenant_1_id},
                               {'id': 'n2',
                                'project_id': tenant_2_id}])
        networks = [getattr(net, 'Network') for net in db_lib.get_networks()]
        project_list = [net.project_id for net in networks]
        self.assertIn(tenant_1_id, project_list)
        self.assertIn(tenant_2_id, project_list)

    def test_get_tenants_with_shared_network_ports(self):
        tenant_1_id = 't1'
        tenant_2_id = 't2'
        utils.create_networks([{'id': 'n1',
                                'project_id': tenant_1_id}])
        utils.create_ports([{'admin_state_up': True,
                             'status': 'ACTIVE',
                             'device_id': 'vm1',
                             'device_owner': 'compute:None',
                             'tenant_id': tenant_2_id,
                             'id': 'p1',
                             'network_id': 'n1',
                             'mac_address': '00:00:00:00:00:01'}])
        tenants = db_lib.get_tenants()
        expected_tenants = [{'project_id': tenant_1_id},
                            {'project_id': tenant_2_id}]
        self.assertItemsEqual(tenants, expected_tenants)

    def test_get_tenants_uniqueness(self):
        tenant_1_id = 't1'
        tenant_2_id = 't2'
        utils.create_networks([{'id': 'n1',
                                'project_id': tenant_1_id},
                               {'id': 'n2',
                                'project_id': tenant_2_id}])
        utils.create_ports([{'admin_state_up': True,
                             'status': 'ACTIVE',
                             'device_id': 'vm1',
                             'device_owner': 'compute:None',
                             'tenant_id': tenant_1_id,
                             'id': 'p1',
                             'network_id': 'n1',
                             'mac_address': '00:00:00:00:00:01'},
                            {'admin_state_up': True,
                             'status': 'ACTIVE',
                             'device_id': 'vm2',
                             'device_owner': 'compute:None',
                             'tenant_id': tenant_2_id,
                             'id': 'p2',
                             'network_id': 'n2',
                             'mac_address': '00:00:00:00:00:02'}])
        tenants = db_lib.get_tenants()
        expected_tenants = [{'project_id': tenant_1_id},
                            {'project_id': tenant_2_id}]
        self.assertItemsEqual(tenants, expected_tenants)

    def test_get_tenants_port_network_union(self):
        tenant_1_id = 't1'
        tenant_2_id = 't2'
        tenant_3_id = 't3'
        tenant_4_id = 't4'
        utils.create_networks([{'id': 'n1',
                                'project_id': tenant_1_id},
                               {'id': 'n2',
                                'project_id': tenant_2_id}])
        utils.create_ports([{'admin_state_up': True,
                             'status': 'ACTIVE',
                             'device_id': 'vm1',
                             'device_owner': 'compute:None',
                             'tenant_id': tenant_3_id,
                             'id': 'p1',
                             'network_id': 'n1',
                             'mac_address': '00:00:00:00:00:01'},
                            {'admin_state_up': True,
                             'status': 'ACTIVE',
                             'device_id': 'vm2',
                             'device_owner': 'compute:None',
                             'tenant_id': tenant_4_id,
                             'id': 'p2',
                             'network_id': 'n2',
                             'mac_address': '00:00:00:00:00:02'}])

        tenants = db_lib.get_tenants()
        expected_tenants = [{'project_id': tenant_1_id},
                            {'project_id': tenant_2_id},
                            {'project_id': tenant_3_id},
                            {'project_id': tenant_4_id}]
        self.assertItemsEqual(tenants, expected_tenants)

    def test_tenant_provisioned(self):
        tenant_1_id = 't1'
        port_1_id = 'p1'
        tenant_2_id = 't2'
        port_2_id = 'p2'
        network_id = 'network-id'
        self.assertFalse(db_lib.tenant_provisioned(tenant_1_id))
        self.assertFalse(db_lib.tenant_provisioned(tenant_2_id))
        utils.create_networks([{'id': network_id,
                                'project_id': tenant_1_id}])
        self.assertTrue(db_lib.tenant_provisioned(tenant_1_id))
        self.assertFalse(db_lib.tenant_provisioned(tenant_2_id))
        utils.create_ports([{'admin_state_up': True,
                             'status': 'ACTIVE',
                             'device_id': 'vm1',
                             'device_owner': 'compute:None',
                             'tenant_id': tenant_1_id,
                             'id': port_1_id,
                             'network_id': network_id,
                             'mac_address': '00:00:00:00:00:01'}])
        self.assertTrue(db_lib.tenant_provisioned(tenant_1_id))
        self.assertFalse(db_lib.tenant_provisioned(tenant_2_id))
        utils.create_ports([{'admin_state_up': True,
                             'status': 'ACTIVE',
                             'device_id': 'vm2',
                             'device_owner': 'compute:None',
                             'tenant_id': tenant_2_id,
                             'id': port_2_id,
                             'network_id': network_id,
                             'mac_address': '00:00:00:00:00:02'}])
        self.assertTrue(db_lib.tenant_provisioned(tenant_1_id))
        self.assertTrue(db_lib.tenant_provisioned(tenant_2_id))
        utils.delete_port(port_1_id)
        utils.delete_port(port_2_id)
        utils.delete_network(network_id)
        self.assertFalse(db_lib.tenant_provisioned(tenant_1_id))
        self.assertFalse(db_lib.tenant_provisioned(tenant_2_id))

    def test_get_segments(self):
        net_id1 = 'n1'
        net_id2 = 'n2'
        utils.create_networks([{'id': net_id1,
                                'project_id': 'p1'},
                               {'id': net_id2,
                                'project_id': ''}])
        utils.create_segments([{'id': 's1',
                                'network_id': net_id1,
                                'network_type': 'vlan',
                                'segmentation_id': 100,
                                'is_dynamic': False},
                               {'id': 's2',
                                'network_id': net_id2,
                                'network_type': 'vlan',
                                'segmentation_id': 200,
                                'is_dynamic': True}])
        segments = db_lib.get_segments()
        netid_list = [seg.network_id for seg in segments]
        self.assertIn(net_id1, netid_list)
        self.assertIn(net_id2, netid_list)

    def test_segment_is_dynamic(self):
        static_segment_id = 's1'
        dynamic_segment_id = 's2'
        utils.create_networks([{'id': 'n1',
                                'project_id': 't1'}])
        utils.create_segments([{'id': static_segment_id,
                                'network_id': 'n1',
                                'network_type': 'vlan',
                                'segmentation_id': 100,
                                'is_dynamic': False},
                               {'id': dynamic_segment_id,
                                'network_id': 'n1',
                                'network_type': 'vlan',
                                'segmentation_id': 200,
                                'is_dynamic': True}])
        self.assertFalse(db_lib.segment_is_dynamic(static_segment_id))
        self.assertTrue(db_lib.segment_is_dynamic(dynamic_segment_id))

    def test_segment_bound(self):
        bound_segment_id = 's1'
        unbound_segment_id = 's2'
        utils.create_networks([{'id': 'n1',
                                'project_id': 't1'}])
        utils.create_segments([{'id': bound_segment_id,
                                'network_id': 'n1',
                                'network_type': 'vlan',
                                'segmentation_id': 100,
                                'is_dynamic': True},
                               {'id': unbound_segment_id,
                                'network_id': 'n1',
                                'network_type': 'vlan',
                                'segmentation_id': 200,
                                'is_dynamic': True}])
        utils.create_ports([{'admin_state_up': True,
                             'status': 'ACTIVE',
                             'device_id': 'vm1',
                             'device_owner': 'compute:None',
                             'binding': {'host': 'host',
                                         'vif_type': 'ovs',
                                         'vnic_type': 'normal'},
                             'tenant_id': 't1',
                             'id': 'p1',
                             'network_id': 'n1',
                             'mac_address': '01:02:03:04:05:06',
                             'binding_levels': [
                                 {'host': 'host',
                                  'segment_id': bound_segment_id,
                                  'level': 0,
                                  'driver': 'arista'}]}])
        self.assertTrue(db_lib.segment_bound(bound_segment_id))
        self.assertFalse(db_lib.segment_bound(unbound_segment_id))

    def test_empty_device_id(self):
        utils.create_networks([{'id': 'n1',
                                'project_id': 't1'}])
        utils.create_segments([{'id': 's1',
                                'network_id': 'n1',
                                'network_type': 'vlan',
                                'segmentation_id': 100,
                                'is_dynamic': True}])
        utils.create_ports([{'admin_state_up': True,
                             'status': 'ERROR',
                             'device_id': '',
                             'device_owner': 'compute:None',
                             'binding': {'host': 'host',
                                         'vif_type': 'other',
                                         'vnic_type': 'baremetal'},
                             'tenant_id': 't1',
                             'id': 'p1',
                             'network_id': 'n1',
                             'mac_address': '01:02:03:04:05:06',
                             'binding_levels': []},
                            {'admin_state_up': True,
                             'status': 'ACTIVE',
                             'device_id': 'bm1',
                             'device_owner': 'compute:None',
                             'binding': {'host': 'host',
                                         'vif_type': 'other',
                                         'vnic_type': 'baremetal'},
                             'tenant_id': 't1',
                             'id': 'p2',
                             'network_id': 'n1',
                             'mac_address': '01:02:03:04:05:07',
                             'binding_levels': [
                                 {'host': 'host',
                                  'segment_id': 's1',
                                  'level': 0,
                                  'driver': 'arista'}]},
                            {'admin_state_up': True,
                             'status': 'ACTIVE',
                             'device_id': 'bm2',
                             'device_owner': 'compute:None',
                             'binding': {'host': 'host',
                                         'vif_type': 'other',
                                         'vnic_type': 'baremetal'},
                             'tenant_id': 't1',
                             'id': 'p3',
                             'network_id': 'n1',
                             'mac_address': '01:02:03:04:05:08',
                             'binding_levels': [
                                 {'host': 'host',
                                  'segment_id': 's1',
                                  'level': 0,
                                  'driver': 'arista'}]}])
        expected_instances = []
        instances = db_lib.get_baremetal_instances(instance_id='')
        self.assertItemsEqual(instances, expected_instances)



# Copyright (c) 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import random
import time

from eventlet import event
from eventlet import greenthread
from six.moves.queue import Empty

from neutron_lib import worker
from oslo_config import cfg
from oslo_log import log as logging

from networking_arista.common import constants as a_const
from networking_arista.ml2 import arista_resources as resources
from networking_arista.ml2.rpc.arista_json import AristaRPCWrapperJSON

LOG = logging.getLogger(__name__)


class AristaSyncWorker(worker.BaseWorker):
    def __init__(self, provision_queue):
        super(AristaSyncWorker, self).__init__(worker_process_count=1)
        self._rpc = AristaRPCWrapperJSON()
        self.provision_queue = provision_queue
        self._thread = None
        self._running = False
        self.done = event.Event()
        self._sync_interval = cfg.CONF.ml2_arista.sync_interval

    def initialize(self):
        self._last_sync_time = 0
        self._cvx_uuid = None
        self._synchronizing_uuid = None
        self._resources_to_update = list()

        self.tenants = resources.Tenants(self._rpc)
        self.networks = resources.Networks(self._rpc)
        self.segments = resources.Segments(self._rpc)
        self.dhcps = resources.Dhcps(self._rpc)
        self.routers = resources.Routers(self._rpc)
        self.vms = resources.Vms(self._rpc)
        self.baremetals = resources.Baremetals(self._rpc)
        self.dhcp_ports = resources.DhcpPorts(self._rpc)
        self.router_ports = resources.RouterPorts(self._rpc)
        self.vm_ports = resources.VmPorts(self._rpc)
        self.baremetal_ports = resources.BaremetalPorts(self._rpc)
        self.port_bindings = resources.PortBindings(self._rpc)

        # Sync order is important because of entity dependencies:
        # PortBinding -> Port -> Instance -> Tenant
        #             -> Segment -> Network -> Tenant
        self.sync_order = [self.tenants,
                           self.networks,
                           self.segments,
                           self.dhcps,
                           self.routers,
                           self.vms,
                           self.baremetals,
                           self.dhcp_ports,
                           self.router_ports,
                           self.vm_ports,
                           self.baremetal_ports,
                           self.port_bindings]

    def _on_done(self, gt, *args, **kwargs):
        self._thread = None
        self._running = False

    def start(self):
        if self._thread is not None:
            LOG.warning('Arista sync loop has already been started')
            return

        LOG.info("Arista sync worker started")
        super(AristaSyncWorker, self).start()
        self.initialize()
        self._running = True
        LOG.info("Spawning Arista sync loop")
        self._thread = greenthread.spawn(self.sync_loop)
        self._thread.link(self._on_done)

    def stop(self, graceful=True):
        if graceful:
            self._running = False
        else:
            self._thread.kill()

    def wait(self):
        return self.done.wait()

    def reset(self):
        self.stop()
        self.wait()
        self.start()

    def get_resource_class(self, resource_type):
        class_map = {a_const.TENANT_RESOURCE: self.tenants,
                     a_const.NETWORK_RESOURCE: self.networks,
                     a_const.SEGMENT_RESOURCE: self.segments,
                     a_const.DHCP_RESOURCE: self.dhcps,
                     a_const.ROUTER_RESOURCE: self.routers,
                     a_const.VM_RESOURCE: self.vms,
                     a_const.BAREMETAL_RESOURCE: self.baremetals,
                     a_const.DHCP_PORT_RESOURCE: self.dhcp_ports,
                     a_const.ROUTER_PORT_RESOURCE: self.router_ports,
                     a_const.VM_PORT_RESOURCE: self.vm_ports,
                     a_const.BAREMETAL_PORT_RESOURCE: self.baremetal_ports,
                     a_const.PORT_BINDING_RESOURCE: self.port_bindings}
        return class_map[resource_type]

    def update_neutron_resource(self, resource):
        LOG.debug("%(pid)s %(action)s %(rtype)s with id %(id)s",
                  {'action': resource.action,
                   'rtype': resource.resource_type,
                   'id': resource.id,
                   'pid': os.getpid()})
        resource_class = self.get_resource_class(resource.resource_type)
        resource_class.update_neutron_resource(resource.id, resource.action)
        for resource_type, resource_id in resource.related_resources:
            LOG.debug("%(pid)s %(action)s requisite %(rtype)s with id %(id)s",
                      {'action': resource.action,
                       'rtype': resource_type,
                       'id': resource_id,
                       'pid': os.getpid()})
            resource_class = self.get_resource_class(resource_type)
            resource_class.update_neutron_resource(resource_id,
                                                   resource.action)

    def force_full_sync(self):
        for resource_type in reversed(self.sync_order):
            resource_type.clear_all_data()

    def port_bindings_missing(self):
        # Check for any DVR port bindings that weren't synced
        current_port_bindings = self.port_bindings.get_neutron_ids()
        self.port_bindings.neutron_data_stale = True
        if current_port_bindings != self.port_bindings.get_neutron_ids():
            return True
        return False

    def check_if_out_of_sync(self):
        cvx_uuid = self._rpc.get_cvx_uuid()
        out_of_sync = False
        if self._cvx_uuid != cvx_uuid:
            LOG.info("%(pid)s Initiating full sync - local uuid %(l_uuid)s"
                     " - cvx uuid %(c_uuid)s",
                     {'l_uuid': self._cvx_uuid,
                      'c_uuid': cvx_uuid,
                      'pid': os.getpid()})
            self.force_full_sync()
            self._synchronizing_uuid = cvx_uuid
            out_of_sync = True
        elif self.port_bindings_missing():
            LOG.info("%(pid)s DVR port bindings out of sync. Triggering sync.",
                     {'pid': os.getpid()})
            out_of_sync = True

        self._last_sync_time = time.time()
        return out_of_sync

    def wait_for_mech_driver_update(self, timeout):
        try:
            resource = self.provision_queue.get(timeout=timeout)
            LOG.info("%(pid)s Processing %(res)s", {'res': resource,
                                                    'pid': os.getpid()})
            self._resources_to_update.append(resource)
        except Empty:
            pass
        return len(self._resources_to_update) > 0

    def wait_for_sync_required(self):
        timeout = (self._sync_interval -
                   (time.time() - self._last_sync_time))
        LOG.info("%(pid)s Arista Sync time %(time)s last sync %(last_sync)s "
                 "timeout %(timeout)s", {'time': time.time(),
                                         'last_sync': self._last_sync_time,
                                         'timeout': timeout,
                                         'pid': os.getpid()})
        if timeout < 0:
            return self.check_if_out_of_sync()
        else:
            return self.wait_for_mech_driver_update(timeout)

    def synchronize_resources(self):
        """Synchronize worker with CVX

        All database queries must occur while the sync lock is held. This
        tightly couples reads with writes and ensures that an older read
        does not result in the last write. Eg:

        Worker 1 reads (P1 created)
        Worder 2 reads (P1 deleted)
        Worker 2 writes (Delete P1 from CVX)
        Worker 1 writes (Create P1 on CVX)

        By ensuring that all reads occur with the sync lock held, we ensure
        that Worker 1 completes its writes before Worker2 is allowed to read.
        A failure to write results in a full resync and purges all reads from
        memory.

        It is also important that we compute resources to sync in reverse sync
        order in order to avoid missing dependencies on creation. Eg:

        If we query in sync order
        1. Query Instances -> I1 isn't there
        2. Query Port table -> Port P1 is there, connected to I1
        3. We send P1 to CVX without sending I1 -> Error raised

        But if we query P1 first:
        1. Query Ports P1 -> P1 is not there
        2. Query Instances -> find I1
        3. We create I1, not P1 -> harmless, mech driver creates P1

        Missing dependencies on deletion will helpfully result in the
        dependent resource not being created:
        1. Query Ports -> P1 is found
        2. Query Instances -> I1 not found
        3. Creating P1 fails on CVX
        """
        # Grab the sync lock
        if not self._rpc.sync_start():
            LOG.info("%(pid)s Failed to grab the sync lock",
                     {'pid': os.getpid()})
            return

        for resource in self._resources_to_update:
            self.update_neutron_resource(resource)
        self._resources_to_update = list()

        # Sync any necessary resources.
        # We delete in reverse order and create in order to ensure that
        # dependent resources  are deleted before the resources they depend
        # on and created after them
        for resource_type in reversed(self.sync_order):
            resource_type.delete_cvx_resources()
        for resource_type in self.sync_order:
            resource_type.create_cvx_resources()

        # Release the sync lock
        self._rpc.sync_end()

        # Update local uuid if this was a full sync
        if self._synchronizing_uuid:
            LOG.info("%(pid)s Full sync for cvx uuid %(uuid)s complete",
                     {'uuid': self._synchronizing_uuid,
                      'pid': os.getpid()})
            self._cvx_uuid = self._synchronizing_uuid
            self._synchronizing_uuid = None

    def sync_loop(self):
        while self._running:
            try:
                sync_required = self.wait_for_sync_required()

                if sync_required:
                    self.synchronize_resources()
            except Exception:
                LOG.exception("%(pid)s Arista Sync failed",
                              {'pid': os.getpid()})
                # Release sync lock if held and at least one full sync was
                # successful with the current cvx uuid
                if (self._rpc.current_sync_name is not None and
                        self._cvx_uuid is not None):
                    self._rpc.sync_end()
                self._cvx_uuid = None
                self._synchronizing_uuid = None

            # Yield to avoid starvation
            greenthread.sleep(random.random())

        self.done.send(True)



# Copyright (c) 2016 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron_lib.api.definitions import portbindings
from neutron_lib.callbacks import events
from neutron_lib.callbacks import registry
from neutron_lib.callbacks import resources
from neutron_lib.plugins import directory
from neutron_lib.services import base as service_base
from oslo_log import helpers as log_helpers
from oslo_log import log as logging

from networking_arista.ml2.security_groups import security_group_sync
from networking_arista.ml2.security_groups import switch_helper


LOG = logging.getLogger(__name__)


class AristaSecurityGroupHandler(
        service_base.ServicePluginBase,
        switch_helper.AristaSecurityGroupSwitchHelper):
    """Security Group Handler for Arista networking hardware.

    Registers for the notification of security group updates.
    Once a notification is recieved, it takes appropriate actions by updating
    Arista hardware appropriately.
    """

    def __init__(self):
        super(AristaSecurityGroupHandler, self).__init__()
        self.initialize_switch_endpoints()
        self.subscribe()
        self.add_worker(security_group_sync.AristaSecurityGroupSyncWorker())

    def get_plugin_description(self):
        return "Arista baremetal security group service plugin"

    @classmethod
    def get_plugin_type(cls):
        return "arista_security_group"

    @log_helpers.log_method_call
    def create_security_group(self, resource, event, trigger, **kwargs):
        sg = kwargs.get('security_group')
        rules = sg['security_group_rules']
        sg_id = sg['id']
        cmds = self.get_create_security_group_commands(sg_id, rules)
        self.run_cmds_on_all_switches(cmds)

    @log_helpers.log_method_call
    def delete_security_group(self, resource, event, trigger, **kwargs):
        sg_id = kwargs.get('security_group_id')
        cmds = self.get_delete_security_group_commands(sg_id)
        self.run_cmds_on_all_switches(cmds)

    @log_helpers.log_method_call
    def create_security_group_rule(self, resource, event, trigger, **kwargs):
        sgr = kwargs.get('security_group_rule')
        sg_id = sgr['security_group_id']
        cmds = self.get_create_security_group_rule_commands(sg_id, sgr)
        self.run_cmds_on_all_switches(cmds)

    @log_helpers.log_method_call
    def delete_security_group_rule(self, resource, event, trigger, **kwargs):
        sgr_id = kwargs.get('security_group_rule_id')
        context = kwargs.get('context')
        plugin = directory.get_plugin()
        sgr = plugin.get_security_group_rule(context, sgr_id)
        sg_id = sgr['security_group_id']
        cmds = self.get_delete_security_group_rule_commands(sg_id, sgr)
        self.run_cmds_on_all_switches(cmds)

    @staticmethod
    def _valid_baremetal_port(port):
        """Check if port is a baremetal port with exactly one security group"""
        if port.get(portbindings.VNIC_TYPE) != portbindings.VNIC_BAREMETAL:
            return False
        sgs = port.get('security_groups', [])
        if len(sgs) == 0:
            # Nothing to do
            return False
        if len(port.get('security_groups', [])) > 1:
            LOG.warning('SG provisioning failed for %(port)s. Only one '
                        'SG may be applied per port.',
                        {'port': port['id']})
            return False
        return True

    @log_helpers.log_method_call
    def apply_security_group(self, resource, event, trigger, **kwargs):
        port = kwargs.get('port')
        if not self._valid_baremetal_port(port):
            return
        # _valid_baremetal_port guarantees we have exactly one SG
        sg_id = port.get('security_groups')[0]
        profile = port.get(portbindings.PROFILE, {})
        self._update_port_group_info(switches=self._get_switches(profile))
        switch_cmds = self.get_apply_security_group_commands(sg_id, profile)
        self.run_per_switch_cmds(switch_cmds)

    @log_helpers.log_method_call
    def remove_security_group(self, resource, event, trigger, **kwargs):
        port = kwargs.get('port')
        if not self._valid_baremetal_port(port):
            return
        # _valid_baremetal_port guarantees we have exactly one SG
        sg_id = port.get('security_groups')[0]
        profile = port.get(portbindings.PROFILE, {})
        self._update_port_group_info(switches=self._get_switches(profile))
        switch_cmds = self.get_remove_security_group_commands(sg_id, profile)
        self.run_per_switch_cmds(switch_cmds)

    def subscribe(self):
        # Subscribe to the events related to security groups and rules
        registry.subscribe(
            self.create_security_group, resources.SECURITY_GROUP,
            events.AFTER_CREATE)
        registry.subscribe(
            self.delete_security_group, resources.SECURITY_GROUP,
            events.AFTER_DELETE)
        registry.subscribe(
            self.create_security_group_rule, resources.SECURITY_GROUP_RULE,
            events.AFTER_CREATE)

        # We need to handle SG rules in before delete to be able to query
        # the db for the rule details
        registry.subscribe(
            self.delete_security_group_rule, resources.SECURITY_GROUP_RULE,
            events.BEFORE_DELETE)

        # Apply SG rules to intfs on AFTER_UPDATE, remove them on AFTER_DELETE
        registry.subscribe(
            self.apply_security_group, resources.PORT, events.AFTER_UPDATE)
        registry.subscribe(
            self.remove_security_group, resources.PORT, events.AFTER_DELETE)
